# -*- coding: UTF-8 -*-

from sqlUnloader import SQLUnloader, CaseInsensitiveDict
from common import logger

class DocumentUnloader(SQLUnloader):
    discountTypes = {1: 'percent', 2: 'summ', 3: 'index', 4: 'price', 5: 'bonus'}
    discountModes = {1: 'automatic', 2: 'byCard', 3: 'fixed', 4: 'byPaymentCard', 5: 'byDiscKit', 
                     6: 'byChange', 7: 'byPayment', 8: 'certificate', 9: 'byCoupon', 10: 'correction', 11: 'fiscalRound'}
    quantMode = {1: 'default', 2: 'manual', 3: 'fromBarCode', 4: 'fromScales', 5: 'external', 6: 'softCheck'}
    bcodeMode = {1: 'manual', 2: 'scanner', 3: 'hotkey', 4: 'selected', 5: 'quickCode', 6: 'complect', 7: 'external', 8: 'mcr', 9: 'softCheck', 10: 'goodsdetector'}

    def __init__(self, connection, log):
        self.cursor = connection.cursor()
        self.tdata = connection.cursor()
        self.tsdata = connection.cursor()
        self.mdata = connection.cursor()
        self.failedmdata = connection.cursor()
        self.cdata = connection.cursor()
        self.discdata = connection.cursor()
        self.bonusdata = connection.cursor()
        self.carddata = connection.cursor()
        self.coupondata = connection.cursor()
        self.deptdata = connection.cursor()
        self.clientdata = connection.cursor()
        self.paymentdata = connection.cursor()
        self.aspectdata = connection.cursor()
        self.buttondata = connection.cursor()
        self.certdata = connection.cursor()
        self.giftdata = connection.cursor()
        self.alcosetdata = connection.cursor()
        self.supplierdata = connection.cursor()
        self.agentsdata = connection.cursor()
        self.consultantdata = connection.cursor()
        self.medicinedata = connection.cursor()
        self.fiscaldocumentdata = connection.cursor()
        self.additionalpricedata = connection.cursor()
        self.log = log

    def produceMDataObject(self, dict_lower):
        """ moneyitem """
        docnum = self.getValue(dict_lower, 'moneyitem.docnum')
        docnum = "" if docnum is None else docnum

        return {
            'dopdata': self.getValue(dict_lower, 'moneyitem.moneyitemid'),
            'acode': self.getValue(dict_lower, 'moneyitem.acode'),
            'userCode': self.getValue(dict_lower, 'moneyitem.scode'),
            'posTime': self.getValue(dict_lower, 'moneyitem.mtime'),
            'opCode': self.getValue(dict_lower, 'moneyitem.opcode'),
            'valCode': self.getValue(dict_lower, 'moneyitem.valcode'),
            'nrate': self.getValue(dict_lower, 'moneyitem.nrate'),
            'bond': self.getValue(dict_lower, 'moneyitem.bond'),
            'bond_quant': self.getValue(dict_lower, 'moneyitem.bond_quant'),
            'vsum': self.getValue(dict_lower, 'moneyitem.vsum'),
            'sumB': self.getValue(dict_lower, 'moneyitem.sumb'),
            'sumn': self.getValue(dict_lower, 'moneyitem.sumn'),
            'sume': self.getValue(dict_lower, 'moneyitem.sume'),
            'docnum': docnum,
            'c_link': self.getValue(dict_lower, 'moneyitem.c_link'),
            'frnum': self.getValue(dict_lower, 'moneyitem.frnum'),
            'cardnum': self.getValue(dict_lower, 'moneyitem.cardnum'),
            'cashcode': self.getValue(dict_lower, 'moneyitem.cashcode'),
            'valutOperation': self.getValue(dict_lower, 'moneyitem.valutoperation'),
            'valName': self.getValue(dict_lower, 'moneyitem.valname'),
            'balance': self.getValue(dict_lower, 'moneyitem.balance'),
            'operationId': self.getValue(dict_lower, 'moneyitem.operationid'),
            'slip': self.getValue(dict_lower, 'moneyitem.slip'),
            'discnumber': self.getValue(dict_lower, 'moneyitem.discnumber'),
            'authcode': self.getValue(dict_lower, 'moneyitem.authcode'),
            'merchantid': self.getValue(dict_lower, 'moneyitem.merchantid'),
            'sourceoperationid': self.getValue(dict_lower, 'moneyitem.sourceoperationid'),
            'paymentmethod': self.getValue(dict_lower, 'moneyitem.paymentmethod'),
            'endcardnum': self.getValue(dict_lower, 'moneyitem.endcardnum'),
            'terminalid': self.getValue(dict_lower, 'moneyitem.terminalid'),
            'additionaldata': self.getValue(dict_lower, 'moneyitem.additionaldata'),

            ################################ ВНИМАНИЕ ################################
            ################### Все поля добавляем в нижнем регистре #################
            #################### Наименование поля такое же как в БД #################
            ##########################################################################

            }

    def produceFailedMDataObject(self, dict_lower):
        """ failedmoneyitem """
        docnum = self.getValue(dict_lower, 'failedmoneyitem.docnum')
        docnum = "" if docnum is None else docnum

        return {
            'dopdata': self.getValue(dict_lower, 'failedmoneyitem.failedmoneyitemid'),
            'userCode': self.getValue(dict_lower, 'failedmoneyitem.scode'),
            'posTime': self.getValue(dict_lower, 'failedmoneyitem.mtime'),
            'opCode': self.getValue(dict_lower, 'failedmoneyitem.opcode'),
            'valCode': self.getValue(dict_lower, 'failedmoneyitem.valcode'),
            'sumB': self.getValue(dict_lower, 'failedmoneyitem.sumb'),
            'docnum': docnum,
            'frnum': self.getValue(dict_lower, 'failedmoneyitem.frnum'),
            'cardnum': self.getValue(dict_lower, 'failedmoneyitem.cardnum'),
            'cashcode': self.getValue(dict_lower, 'failedmoneyitem.cashcode'),
            'slip': self.getValue(dict_lower, 'failedmoneyitem.slip'),
            'valutOperation': self.getValue(dict_lower, 'failedmoneyitem.valutoperation'),
            'valName': self.getValue(dict_lower, 'failedmoneyitem.valname'),
            'merchantid': self.getValue(dict_lower, 'failedmoneyitem.merchantid'),
            'endcardnum': self.getValue(dict_lower, 'failedmoneyitem.endcardnum'),
            'terminalid': self.getValue(dict_lower, 'failedmoneyitem.terminalid'),
            'additionaldata': self.getValue(dict_lower, 'failedmoneyitem.additionaldata'),

            ################################ ВНИМАНИЕ ################################
            ################### Все поля добавляем в нижнем регистре #################
            #################### Наименование поля такое же как в БД #################
            ##########################################################################

            }

    def produceTDataObject(self, dict_lower):
        """ goodsitem """
        return {
            'dopdata': self.getValue(dict_lower, 'goodsitem.goodsitemid'),
            'deptCode': self.getValue(dict_lower, 'goodsitem.deptcode'),
            'userCode': self.getValue(dict_lower, 'goodsitem.scode'),
            'posTime': self.getValue(dict_lower, 'goodsitem.ttime'),
            'opCode': self.getValue(dict_lower, 'goodsitem.opcode'),
            'status': self.getValue(dict_lower, 'goodsitem.status'),
            'barCode': self.getValue(dict_lower, 'goodsitem.bcode'),
            'name': self.getValue(dict_lower, 'goodsitem.name'),
            'articul': self.getValue(dict_lower, 'goodsitem.articul'),
            'measureCode': self.getValue(dict_lower, 'goodsitem.measure'),
            'bcodeMode': self.bcodeMode.get(self.getValue(dict_lower, 'goodsitem.bcode_mode'), 'unknown'),#decrypt
            'bcode_main': self.getValue(dict_lower, 'goodsitem.bcode_main'),#decrypt
            'quant': self.getValue(dict_lower, 'goodsitem.bquant'),
            'quantMode': self.quantMode.get(self.getValue(dict_lower, 'goodsitem.bquant_mode'), 'unknown'),#decrypt
            'ost_modif': self.getValue(dict_lower, 'goodsitem.ost_modif'),
            'accountingQuant': self.getValue(dict_lower, 'goodsitem.cquant'),
            'priceType': self.getValue(dict_lower, 'goodsitem.pricetype'),
            'pricevcode': self.getValue(dict_lower, 'goodsitem.pricevcode'),
            'price': self.getValue(dict_lower, 'goodsitem.price'),
            'minPrice': self.getValue(dict_lower, 'goodsitem.minprice'),
            'finalPrice': self.getValue(dict_lower, 'goodsitem.pricei'),
            'priceMode': self.getValue(dict_lower, 'goodsitem.price_mode'),#decrypt
            'priceIndex': self.getValue(dict_lower, 'goodsitem.tindex'),
            'disc_perc': self.getValue(dict_lower, 'goodsitem.disc_perc'),
            'disc_abs': self.getValue(dict_lower, 'goodsitem.disc_abs'),
            'modSum': self.getValue(dict_lower, 'goodsitem.sumi'),
            'posSum': self.getValue(dict_lower, 'goodsitem.sumb'),
            'baseSum': self.getValue(dict_lower, 'goodsitem.sumn'),
            'sume': self.getValue(dict_lower, 'goodsitem.sume'),
            'vatcode1': self.getValue(dict_lower, 'goodsitem.vatcode1'),
            'vatrate1': self.getValue(dict_lower, 'goodsitem.vatrate1'),
            'vatsum1': self.getValue(dict_lower, 'goodsitem.vatsum1'),
            'vatcode2': self.getValue(dict_lower, 'goodsitem.vatcode2'),
            'vatrate2': self.getValue(dict_lower, 'goodsitem.vatrate2'),
            'vatsum2': self.getValue(dict_lower, 'goodsitem.vatsum2'),
            'vatcode3': self.getValue(dict_lower, 'goodsitem.vatcode3'),
            'vatrate3': self.getValue(dict_lower, 'goodsitem.vatrate3'),
            'vatsum3': self.getValue(dict_lower, 'goodsitem.vatsum3'),
            'vatcode4': self.getValue(dict_lower, 'goodsitem.vatcode4'),
            'vatrate4': self.getValue(dict_lower, 'goodsitem.vatrate4'),
            'vatsum4': self.getValue(dict_lower, 'goodsitem.vatsum4'),
            'vatcode5': self.getValue(dict_lower, 'goodsitem.vatcode5'),
            'vatrate5': self.getValue(dict_lower, 'goodsitem.vatrate5'),
            'vatsum5': self.getValue(dict_lower, 'goodsitem.vatsum5'),
            'docnum': self.getValue(dict_lower, 'goodsitem.docnum'),
            'c_link': self.getValue(dict_lower, 'goodsitem.c_link'),
            'inventCode': self.getValue(dict_lower, 'goodsitem.code'),
            'posNum': self.getValue(dict_lower, 'goodsitem.posnum'),
            'frnum': self.getValue(dict_lower, 'goodsitem.frnum'),
            'extendetoptions': self.getValue(dict_lower, 'goodsitem.extendetoptions'),
            'opid': self.getValue(dict_lower, 'goodsitem.opid'),
            'buttonid': self.getValue(dict_lower, 'goodsitem.buttonid'),
            'paymentitemid': self.getValue(dict_lower, 'goodsitem.paymentitemid'),
            'departmentid': self.getValue(dict_lower, 'goodsitem.departmentid'),
            'taramode': self.getValue(dict_lower, 'goodsitem.taramode'),
            'taracapacity': self.getValue(dict_lower, 'goodsitem.taracapacity'),
            'cashcode': self.getValue(dict_lower, 'goodsitem.cashcode'),
            'extdocid': self.getValue(dict_lower, 'goodsitem.extdocid'),
            'excisemark': self.getValue(dict_lower, 'goodsitem.excisemark'),
            'inn': self.getValue(dict_lower, 'goodsitem.inn'),
            'kpp': self.getValue(dict_lower, 'goodsitem.kpp'),
            'alcoholPercent': self.getValue(dict_lower, 'goodsitem.alcoholpercent'),
            'consultantid': self.getValue(dict_lower, 'goodsitem.consultantid'),
            'additionalexcisemark': self.getValue(dict_lower, 'goodsitem.additionalexcisemark'),
            'alcocode': self.getValue(dict_lower, 'goodsitem.alcocode'),
            'packingprice': self.getValue(dict_lower, 'goodsitem.packingprice'),
            'aspectvaluesetcode': self.getValue(dict_lower, 'goodsitem.aspectvaluesetcode'),
            'additionaldata': self.getValue(dict_lower, 'goodsitem.additionaldata'),
            'aspectschemecode': self.getValue(dict_lower, 'goodsitem.aspectschemecode'),
            'additionalbarcode': self.getValue(dict_lower, 'goodsitem.additionalbarcode'),
            'alctypecode': self.getValue(dict_lower, 'goodsitem.alctypecode'),
            'tags': self.getValue(dict_lower, 'goodsitem.tags'),
            'paymentobject': self.getValue(dict_lower, 'goodsitem.paymentobject'),
            'paymentmethod': self.getValue(dict_lower, 'goodsitem.paymentmethod'),
            'minretailprice': self.getValue(dict_lower, 'goodsitem.minretailprice'),
            'manufacturercountrycode': self.getValue(dict_lower, 'goodsitem.manufacturercountrycode'),
            'customsdeclarationnumber': self.getValue(dict_lower, 'goodsitem.customsdeclarationnumber'),
            'pricedoctype': self.getValue(dict_lower, 'goodsitem.pricedoctype'),
            'prepackaged': self.getValue(dict_lower, 'goodsitem.prepackaged'),
            'excisetype': self.getValue(dict_lower, 'goodsitem.excisetype'),
            'ntin': self.getValue(dict_lower, 'goodsitem.ntin'),
            'checkmarkresult': self.getValue(dict_lower, 'goodsitem.checkmarkresult'),
            'markingcode': self.getValue(dict_lower, 'goodsitem.markingcode'),
            'inputrawdata': self.getValue(dict_lower, 'goodsitem.inputrawdata'),


            ################################ ВНИМАНИЕ ################################
            ################### Все поля добавляем в нижнем регистре #################
            #################### Наименование поля такое же как в БД #################
            ##########################################################################

            }

    def produceTSDataObject(self, dict_lower):
        """ stornogoodsitem """
        return {
            'dopdata': self.getValue(dict_lower, 'stornogoodsitem.stornogoodsitemid'),
            'deptCode': self.getValue(dict_lower, 'stornogoodsitem.deptcode'),
            'userCode': self.getValue(dict_lower, 'stornogoodsitem.scode'),
            'posTime': self.getValue(dict_lower, 'stornogoodsitem.tstime'),
            'opCode': self.getValue(dict_lower, 'stornogoodsitem.opcode'),
            'status': self.getValue(dict_lower, 'stornogoodsitem.status'),
            'barCode': self.getValue(dict_lower, 'stornogoodsitem.bcode'),
            'name': self.getValue(dict_lower, 'stornogoodsitem.name'),
            'articul': self.getValue(dict_lower, 'stornogoodsitem.articul'),
            'measureCode': self.getValue(dict_lower, 'stornogoodsitem.measure'),
            'bcodeMode': self.bcodeMode.get(self.getValue(dict_lower, 'stornogoodsitem.bcode_mode'), 'unknown'),#decrypt
            'bcode_main': self.getValue(dict_lower, 'stornogoodsitem.bcode_main'),#decrypt
            'quant': self.getValue(dict_lower, 'stornogoodsitem.bquant'),
            'quantMode': self.quantMode.get(self.getValue(dict_lower, 'stornogoodsitem.bquant_mode'), 'unknown'),#decrypt
            'ost_modif': self.getValue(dict_lower, 'stornogoodsitem.ost_modif'),
            'accountingQuant': self.getValue(dict_lower, 'stornogoodsitem.cquant'),
            'priceType': self.getValue(dict_lower, 'stornogoodsitem.pricetype'),
            'pricevcode': self.getValue(dict_lower, 'stornogoodsitem.pricevcode'),
            'price': self.getValue(dict_lower, 'stornogoodsitem.price'),
            'minPrice': self.getValue(dict_lower, 'stornogoodsitem.minprice'),
            'finalPrice': self.getValue(dict_lower, 'stornogoodsitem.pricei'),
            'priceMode': self.getValue(dict_lower, 'stornogoodsitem.price_mode'),#decrypt
            'priceIndex': self.getValue(dict_lower, 'stornogoodsitem.tsindex'),
            'disc_perc': self.getValue(dict_lower, 'stornogoodsitem.disc_perc'),
            'disc_abs': self.getValue(dict_lower, 'stornogoodsitem.disc_abs'),
            'modSum': self.getValue(dict_lower, 'stornogoodsitem.sumi'),
            'posSum': self.getValue(dict_lower, 'stornogoodsitem.sumb'),
            'baseSum': self.getValue(dict_lower, 'stornogoodsitem.sumn'),
            'sume': self.getValue(dict_lower, 'stornogoodsitem.sume'),
            'vatcode1': self.getValue(dict_lower, 'stornogoodsitem.vatcode1'),
            'vatrate1': self.getValue(dict_lower, 'stornogoodsitem.vatrate1'),
            'vatsum1': self.getValue(dict_lower, 'stornogoodsitem.vatsum1'),
            'vatcode2': self.getValue(dict_lower, 'stornogoodsitem.vatcode2'),
            'vatrate2': self.getValue(dict_lower, 'stornogoodsitem.vatrate2'),
            'vatsum2': self.getValue(dict_lower, 'stornogoodsitem.vatsum2'),
            'vatcode3': self.getValue(dict_lower, 'stornogoodsitem.vatcode3'),
            'vatrate3': self.getValue(dict_lower, 'stornogoodsitem.vatrate3'),
            'vatsum3': self.getValue(dict_lower, 'stornogoodsitem.vatsum3'),
            'vatcode4': self.getValue(dict_lower, 'stornogoodsitem.vatcode4'),
            'vatrate4': self.getValue(dict_lower, 'stornogoodsitem.vatrate4'),
            'vatsum4': self.getValue(dict_lower, 'stornogoodsitem.vatsum4'),
            'vatcode5': self.getValue(dict_lower, 'stornogoodsitem.vatcode5'),
            'vatrate5': self.getValue(dict_lower, 'stornogoodsitem.vatrate5'),
            'vatsum5': self.getValue(dict_lower, 'stornogoodsitem.vatsum5'),
            'docnum': self.getValue(dict_lower, 'stornogoodsitem.docnum'),
            'c_link': self.getValue(dict_lower, 'stornogoodsitem.c_link'),
            'inventCode': self.getValue(dict_lower, 'stornogoodsitem.code'),
            'posNum': self.getValue(dict_lower, 'stornogoodsitem.posnum'),
            'timeStorno': self.getValue(dict_lower, 'stornogoodsitem.timestorno'),
            'frnum': self.getValue(dict_lower, 'stornogoodsitem.frnum'),
            'extendetoptions': self.getValue(dict_lower, 'stornogoodsitem.extendetoptions'),
            'opid': self.getValue(dict_lower, 'stornogoodsitem.opid'),
            'buttonid': self.getValue(dict_lower, 'stornogoodsitem.buttonid'),
            'paymentitemid': self.getValue(dict_lower, 'stornogoodsitem.paymentitemid'),
            'departmentid': self.getValue(dict_lower, 'stornogoodsitem.departmentid'),
            'taramode': self.getValue(dict_lower, 'stornogoodsitem.taramode'),
            'taracapacity': self.getValue(dict_lower, 'stornogoodsitem.taracapacity'),
            'cashcode': self.getValue(dict_lower, 'stornogoodsitem.cashcode'),
            'extdocid': self.getValue(dict_lower, 'stornogoodsitem.extdocid'),
            'excisemark': self.getValue(dict_lower, 'stornogoodsitem.excisemark'),
            'inn': self.getValue(dict_lower, 'stornogoodsitem.inn'),
            'kpp': self.getValue(dict_lower, 'stornogoodsitem.kpp'),
            'alcoholPercent': self.getValue(dict_lower, 'stornogoodsitem.alcoholpercent'),
            'consultantid': self.getValue(dict_lower, 'stornogoodsitem.consultantid'),
            'additionalexcisemark': self.getValue(dict_lower, 'stornogoodsitem.additionalexcisemark'),
            'packingprice': self.getValue(dict_lower, 'stornogoodsitem.packingprice'),
            'aspectvaluesetcode': self.getValue(dict_lower, 'stornogoodsitem.aspectvaluesetcode'),
            'aspectschemecode': self.getValue(dict_lower, 'stornogoodsitem.aspectschemecode'),
            'additionalbarcode': self.getValue(dict_lower, 'stornogoodsitem.additionalbarcode'),
            'alctypecode': self.getValue(dict_lower, 'stornogoodsitem.alctypecode'),
            'paymentobject': self.getValue(dict_lower, 'stornogoodsitem.paymentobject'),
            'paymentmethod': self.getValue(dict_lower, 'stornogoodsitem.paymentmethod'),
            'minretailprice': self.getValue(dict_lower, 'stornogoodsitem.minretailprice'),
            'tags': self.getValue(dict_lower, 'stornogoodsitem.tags'),
            'manufacturercountrycode': self.getValue(dict_lower, 'stornogoodsitem.manufacturercountrycode'),
            'customsdeclarationnumber': self.getValue(dict_lower, 'stornogoodsitem.customsdeclarationnumber'),
            'pricedoctype': self.getValue(dict_lower, 'stornogoodsitem.pricedoctype'),
            'prepackaged': self.getValue(dict_lower, 'stornogoodsitem.prepackaged'),
            'excisetype': self.getValue(dict_lower, 'stornogoodsitem.excisetype'),
            'ntin': self.getValue(dict_lower, 'stornogoodsitem.ntin'),
            'checkmarkresult': self.getValue(dict_lower, 'stornogoodsitem.checkmarkresult'),
            'markingcode': self.getValue(dict_lower, 'stornogoodsitem.markingcode'),
            'inputrawdata': self.getValue(dict_lower, 'stornogoodsitem.inputrawdata'),


            ################################ ВНИМАНИЕ ################################
            ################### Все поля добавляем в нижнем регистре #################
            #################### Наименование поля такое же как в БД #################
            ##########################################################################

            }

    def produceCDataObject(self, dict_lower):
        """ document """
        return {
            'dopdata': self.getValue(dict_lower, 'document.documentid'),
            'docNum': self.getValue(dict_lower, 'document.checknum'),
            'docType': self.getValue(dict_lower, 'document.doctype'),
            'deptCode': self.getValue(dict_lower, 'document.dept'),
            'userCode': self.getValue(dict_lower, 'document.scode'),
            'cashCode': self.getValue(dict_lower, 'document.cashcode'),
            'shift': self.getValue(dict_lower, 'workshift.shiftnum'),
            'timeBeg': self.getValue(dict_lower, 'document.time_beg'),
            'timeEnd': self.getValue(dict_lower, 'document.time_end'),
            'vbrate': self.getValue(dict_lower, 'document.vbrate'),
            'verate': self.getValue(dict_lower, 'document.verate'),
            'baseSum': self.getValue(dict_lower, 'document.sum1'),
            'modSum': self.getValue(dict_lower, 'document.sum2'),
            'sum2m': self.getValue(dict_lower, 'document.sum2m'),
            'summode': self.getValue(dict_lower, 'document.summode'),
            'sumtype': self.getValue(dict_lower, 'document.sumtype'),
            'disc_perc': self.getValue(dict_lower, 'document.disc_perc'),
            'disc_abs': self.getValue(dict_lower, 'document.disc_abs'),
            'docSum': self.getValue(dict_lower, 'document.sumb'),
            'sumn': self.getValue(dict_lower, 'document.sumn'),
            'sume': self.getValue(dict_lower, 'document.sume'),
            'vatsum': self.getValue(dict_lower, 'document.vatsum'),
            'c_link': self.getValue(dict_lower, 'document.c_link'),
            'closed': self.getValue(dict_lower, 'document.closed'),
            'shiftType': self.getValue(dict_lower, 'workshift.shifttype'),
            'opid': self.getValue(dict_lower, 'document.opid'),
            'dateincrement': self.getValue(dict_lower, 'document.dateincrement'),
            'buttonid': self.getValue(dict_lower, 'document.buttonid'),
            'linkeddocumentid': self.getValue(dict_lower, 'document.linkeddocumentid'),
            'departmentid': self.getValue(dict_lower, 'document.departmentid'),
            'clientitemid': self.getValue(dict_lower, 'document.clientitemid'),
            'identifier': self.getValue(dict_lower, 'document.identifier'),
            'actorCode': self.getValue(dict_lower, 'document.actorcode'),
            'moneyouttype': self.getValue(dict_lower, 'document.moneyouttype'),
            'urlEgais': self.getValue(dict_lower, 'document.url_egais'),
            'digitalSignatureEgais': self.getValue(dict_lower, 'document.digital_signature_egais'),
            'noPdfUrlEgais': self.getValue(dict_lower, 'document.nopdfUrlEgais'),
            'noPdfDigitalSignatureEgais': self.getValue(dict_lower, 'document.nopdfDigitalSignatureEgais'),
            'frDocCopy': self.getValue(dict_lower, 'document.frdoccopy'),
            'customerAddress': self.getValue(dict_lower, 'document.customeraddress'),
            'closeWithoutPrint': self.getValue(dict_lower, 'document.closewithoutprint'),
            'sourceidentifier': self.getValue(dict_lower, 'document.sourceidentifier'),
            'backReason': self.getValue(dict_lower, 'document.backreason'),
            'fiscalIdentifier': self.getValue(dict_lower, 'document.fiscalidentifier'),
            'correctionReason': self.getValue(dict_lower, 'document.correctionreason'),
            'correctionSourceDocNum': self.getValue(dict_lower, 'document.correctionsourcedocnum'),
            'correctionSourceDocDate': self.getValue(dict_lower, 'document.correctionsourcedocdate'),
            'correctionType': self.getValue(dict_lower, 'document.correctiontype'),
            'waybillPrinted': self.getValue(dict_lower, 'document.waybillprinted'),
            'sumcash': self.getValue(dict_lower, 'document.sumcash'),
            'rtext': self.getValue(dict_lower, 'document.rtext'),
            'frdocnum': self.getValue(dict_lower, 'document.frdocnum'),
            'waybillNumber': self.getValue(dict_lower, 'document.waybillnumber'),
            'sumnoncash': self.getValue(dict_lower, 'document.sumnoncash'),
            'sumother': self.getValue(dict_lower, 'document.sumother'),
            'cashType': self.getValue(dict_lower, 'document.cashtype'),

            ################################ ВНИМАНИЕ ################################
            ################### Все поля добавляем в нижнем регистре #################
            #################### Наименование поля такое же как в БД #################
            ##########################################################################
        }

    def produceDiscDataObject(self, dict_lower):
        return {
                'discitemid': self.getValue(dict_lower, 'discitem.discitemid'),
                'goodsitemid': self.getValue(dict_lower, 'discitem.goodsitemid'),
                'positionnum': self.getValue(dict_lower, 'discitem.positionnum'),
                'userCode': self.getValue(dict_lower, 'discitem.scode'),
                'posTime': self.getValue(dict_lower, 'discitem.dtime'),
                'isPositionDiscount': self.getValue(dict_lower, 'discitem.ispositiondiscount') == 1,
                'discNumber': self.getValue(dict_lower, 'discitem.discnumber'),
                'discMode': self.discountModes.get(self.getValue(dict_lower, 'discitem.discmode'), 'unknown'),
                'discType': self.discountTypes.get(self.getValue(dict_lower, 'discitem.disctype'), 'unknown'),
                'discSize': self.getValue(dict_lower, 'discitem.discsize'),
                'discSum': self.getValue(dict_lower, 'discitem.discsum'),
                'discName': self.getValue(dict_lower, 'discitem.discname'),
                'discCode': self.getValue(dict_lower, 'discitem.disccode'),
                'discScheme': self.getValue(dict_lower, 'discitem.discscheme'),
                'checkSum': self.getValue(dict_lower, 'discitem.checksum'),
                'accumulateSum': self.getValue(dict_lower, 'discitem.accumulatesum'),
                'cardNumber': self.getValue(dict_lower, 'discitem.cardnumber'),
                'cardName': self.getValue(dict_lower, 'discitem.cardname'),
                'cardType': self.getValue(dict_lower, 'discitem.cardtype'),
                'carditemid': self.getValue(dict_lower, 'discitem.carditemid'),
                'cashcode': self.getValue(dict_lower, 'discitem.cashcode'),
                'minpriceignored': self.getValue(dict_lower, 'discitem.minpriceignored'),
                'discsystem': self.getValue(dict_lower, 'discitem.discsystem'),
                'sessionid': self.getValue(dict_lower, 'discitem.sessionid'),

            ################################ ВНИМАНИЕ ################################
            ################### Все поля добавляем в нижнем регистре #################
            #################### Наименование поля такое же как в БД #################
            ##########################################################################

                }

    def produceAdditionalPriceDataObject(self, dict_lower):
        return {
                'additionalpriceid': self.getValue(dict_lower, 'additionalprice.additionalpriceid'),
                'discitemid': self.getValue(dict_lower, 'additionalprice.discitemid'),
                'name': self.getValue(dict_lower, 'additionalprice.name'),
                'documentid': self.getValue(dict_lower, 'additionalprice.documentid'),
                'effectivedate': self.getValue(dict_lower, 'additionalprice.effectivedate'),

            ################################ ВНИМАНИЕ ################################
            ################### Все поля добавляем в нижнем регистре #################
            #################### Наименование поля такое же как в БД #################
            ##########################################################################

                }


    def produceBonusDataObject(self, dict_lower):
        return {
                'bonusitemid': self.getValue(dict_lower, 'bonusitem.bonusitemid'),
                'documentid': self.getValue(dict_lower, 'bonusitem.documentid'),
                'goodsitemid': self.getValue(dict_lower, 'bonusitem.goodsitemid'),
                'posnum': self.getValue(dict_lower, 'bonusitem.posnum'),
                'opCode': self.getValue(dict_lower, 'bonusitem.opcode'),
                'cardNumber': self.getValue(dict_lower, 'bonusitem.cardnumber'),
                'amount': self.getValue(dict_lower, 'bonusitem.amount'),
                'posTime': self.getValue(dict_lower, 'bonusitem.btime'),
                'valCode': self.getValue(dict_lower, 'bonusitem.valcode'),
                'initialSum': self.getValue(dict_lower, 'bonusitem.initialsum'),
                'scode': self.getValue(dict_lower, 'bonusitem.scode'),
                'cashcode': self.getValue(dict_lower, 'bonusitem.cashcode'),
                'campaignid': self.getValue(dict_lower, 'bonusitem.campaignid'),
                'ispositionbonus': self.getValue(dict_lower, 'bonusitem.ispositionbonus'),
                'bonusnumber': self.getValue(dict_lower, 'bonusitem.bonusnumber'),
                'assettimebeg': self.getValue(dict_lower, 'bonusitem.assettimebeg'),
                'assettimeend': self.getValue(dict_lower, 'bonusitem.assettimeend'),
                'weight': self.getValue(dict_lower, 'bonusitem.weight'),
                'campaignname': self.getValue(dict_lower, 'bonusitem.campaignname'),
                'transactionid': self.getValue(dict_lower, 'bonusitem.transactionid'),
                'discountcode': self.getValue(dict_lower, 'bonusitem.discountcode')

            ################################ ВНИМАНИЕ ################################
            ################### Все поля добавляем в нижнем регистре #################
            #################### Наименование поля такое же как в БД #################
            ##########################################################################

                }

    def produceCardDataObject(self, dict_lower):
        return {
                'carditemid': self.getValue(dict_lower, 'carditem.carditemid'),
                'documentid': self.getValue(dict_lower, 'carditem.documentid'),
                'idCard': self.getValue(dict_lower, 'carditem.idcard'),
                'idCardGroup': self.getValue(dict_lower, 'carditem.idcardgroup'),
                'idClient': self.getValue(dict_lower, 'carditem.idclient'),
                'number': self.getValue(dict_lower, 'carditem.number'),
                'validityDateBeg': self.getValue(dict_lower, 'carditem.validitydatebeg'),
                'validityDateEnd': self.getValue(dict_lower, 'carditem.validitydateend'),
                'cardSum': self.getValue(dict_lower, 'carditem.cardSum'),
                'nameCardGroup': self.getValue(dict_lower, 'carditem.namecardgroup'),
                'textCardGroup': self.getValue(dict_lower, 'carditem.textcardgroup'),
                'cardMode': self.getValue(dict_lower, 'carditem.cardmode'),
                'prefixBeg': self.getValue(dict_lower, 'carditem.prefixbeg'),
                'prefixEnd': self.getValue(dict_lower, 'carditem.prefixend'),
                'lenBeg': self.getValue(dict_lower, 'carditem.lenbeg'),
                'lenEnd': self.getValue(dict_lower, 'carditem.lenend'),
                'regexPattern': self.getValue(dict_lower, 'carditem.regexpattern'),
                'inputMask': self.getValue(dict_lower, 'carditem.inputmask'),
                'regexRule': self.getValue(dict_lower, 'carditem.regexrule'),
                'valutCode': self.getValue(dict_lower, 'carditem.valutcode'),
                'discountType': self.getValue(dict_lower, 'carditem.discounttype'),
                'discountValue': self.getValue(dict_lower, 'carditem.discountvalue'),
                'nameClient': self.getValue(dict_lower, 'carditem.nameclient'),
                'textClient': self.getValue(dict_lower, 'carditem.textclient'),
                'sex': self.getValue(dict_lower, 'carditem.sex'),
                'birthday': self.getValue(dict_lower, 'carditem.birthday'),
                'specialDate1': self.getValue(dict_lower, 'carditem.specialdate1'),
                'specialDate2': self.getValue(dict_lower, 'carditem.specialdate2'),
                'specialDate3': self.getValue(dict_lower, 'carditem.specialdate3'),
                'specialDate1Name': self.getValue(dict_lower, 'carditem.specialdate1name'),
                'specialDate2Name': self.getValue(dict_lower, 'carditem.specialdate2name'),
                'specialDate3Name': self.getValue(dict_lower, 'carditem.specialdate3name'),
                'zipcode': self.getValue(dict_lower, 'carditem.zipcode'),
                'address': self.getValue(dict_lower, 'carditem.address'),
                'email': self.getValue(dict_lower, 'carditem.email'),
                'webpage': self.getValue(dict_lower, 'carditem.webpage'),
                'phoneNumber': self.getValue(dict_lower, 'carditem.phonenumber'),
                'inn': self.getValue(dict_lower, 'carditem.inn'),
                'document': self.getValue(dict_lower, 'carditem.document'),
                'okpo': self.getValue(dict_lower, 'carditem.okpo'),
                'okpd': self.getValue(dict_lower, 'carditem.okpd'),
                'occupation': self.getValue(dict_lower, 'carditem.occupation'),
                'childrenCount': self.getValue(dict_lower, 'carditem.childrencount'),
                'extendedOptions': self.getValue(dict_lower, 'carditem.extendedoptions'),
                'posTime': self.getValue(dict_lower, 'carditem.itemtime'),
                'itemorder': self.getValue(dict_lower, 'carditem.itemorder'),
                'inputSource': self.getValue(dict_lower, 'carditem.inputsource'),
                'bonusBalance': self.getValue(dict_lower, 'carditem.bonusbalance'),
                'positionnumber': self.getValue(dict_lower, 'carditem.positionnumber'),
                'clientitemid': self.getValue(dict_lower, 'carditem.clientitemid'),
                'cashcode': self.getValue(dict_lower, 'carditem.cashcode'),
                'cardstatus': self.getValue(dict_lower, 'carditem.cardstatus'),
                'rawdata': self.getValue(dict_lower, 'carditem.rawdata'),
                'verificationType': self.getValue(dict_lower, 'carditem.verificationtype'),
                'bonusbalanceinactive': self.getValue(dict_lower, 'carditem.bonusbalanceinactive'),
                'purchases': self.getValue(dict_lower, 'carditem.purchases'),
                'operationinfo': self.getValue(dict_lower, 'carditem.operationinfo'),
                'hash': self.getValue(dict_lower, 'carditem.hash'),
                'options': self.getValue(dict_lower, 'carditem.options'),

            ################################ ВНИМАНИЕ ################################
            ################### Все поля добавляем в нижнем регистре #################
            #################### Наименование поля такое же как в БД #################
            ##########################################################################

                }

    def produceCouponDataObject(self, dict_lower):
        return {
                'couponid': self.getValue(dict_lower, 'couponitem.couponid'),
                'carditemid': self.getValue(dict_lower, 'couponitem.carditemid'),
                'number': self.getValue(dict_lower, 'couponitem.number'),
                'awardvalue': self.getValue(dict_lower, 'couponitem.awardvalue'),
                'offername': self.getValue(dict_lower, 'couponitem.offername'),
                'couponcondition': self.getValue(dict_lower, 'couponitem.couponcondition'),
                'datebegin': self.getValue(dict_lower, 'couponitem.datebegin'),
                'dateexpiration': self.getValue(dict_lower, 'couponitem.dateexpiration'),
                'dateused': self.getValue(dict_lower, 'couponitem.dateused'),
                'status': self.getValue(dict_lower, 'couponitem.status'),
                'type': self.getValue(dict_lower, 'couponitem.type'),
                'documentid': self.getValue(dict_lower, 'couponitem.documentid'),
                'campaignid': self.getValue(dict_lower, 'couponitem.campaignid'),
                'campaignname': self.getValue(dict_lower, 'couponitem.campaignname'),
                'generatednumber': self.getValue(dict_lower, 'couponitem.generatednumber'),

            ################################ ВНИМАНИЕ ################################
            ################### Все поля добавляем в нижнем регистре #################
            #################### Наименование поля такое же как в БД #################
            ##########################################################################

        }

    def produceClientDataObject(self, dict_lower):
        return{
                 'clientitemid': self.getValue(dict_lower, 'client.clientitemid'),
                 'idclient': self.getValue(dict_lower, 'client.idclient'),
                 'name': self.getValue(dict_lower, 'client.name'),
                 'text': self.getValue(dict_lower, 'client.text'),
                 'sex': self.getValue(dict_lower, 'client.sex'),
                 'birthday': self.getValue(dict_lower, 'client.birthday'),
                 'specialdate1': self.getValue(dict_lower, 'client.specialdate1'),
                 'specialdate2': self.getValue(dict_lower, 'client.specialdate2'),
                 'specialdate3': self.getValue(dict_lower, 'client.specialdate3'),
                 'specialdate1name': self.getValue(dict_lower, 'client.specialdate1name'),
                 'specialdate2name': self.getValue(dict_lower, 'client.specialdate2name'),
                 'specialdate3name': self.getValue(dict_lower, 'client.specialdate3name'),
                 'zipcode': self.getValue(dict_lower, 'client.zipcode'),
                 'address': self.getValue(dict_lower, 'client.address'),
                 'email': self.getValue(dict_lower, 'client.email'),
                 'webpage': self.getValue(dict_lower, 'client.webpage'),
                 'phonenumber': self.getValue(dict_lower, 'client.phonenumber'),
                 'inn': self.getValue(dict_lower, 'client.inn'),
                 'document': self.getValue(dict_lower, 'client.document'),
                 'okpo': self.getValue(dict_lower, 'client.okpo'),
                 'okpd': self.getValue(dict_lower, 'client.okpd'),
                 'occupation': self.getValue(dict_lower, 'client.occupation'),
                 'childrencount': self.getValue(dict_lower, 'client.childrencount'),
                 'extendedoptions': self.getValue(dict_lower, 'client.extendedoptions'),
                 'cashcode': self.getValue(dict_lower, 'client.cashcode'),
                 'organizationcode': self.getValue(dict_lower, 'client.organizationcode'),
                 'checkwassenttoemail': self.getValue(dict_lower, 'client.checkwassenttoemail'),

            ################################ ВНИМАНИЕ ################################
            ################### Все поля добавляем в нижнем регистре #################
            #################### Наименование поля такое же как в БД #################
            ##########################################################################

               }

    def produceDeptDataObject(self,dict_lower):
        return{
               'departmentid': self.getValue(dict_lower, 'department.departmentid'),
               'departmentcode': self.getValue(dict_lower, 'department.departmentcode'),
               'departmentname': self.getValue(dict_lower, 'department.departmentname'),
               'clientitemid': self.getValue(dict_lower, 'department.clientitemid'),
               'documentprefix': self.getValue(dict_lower, 'department.documentprefix'),
               'documentnumber': self.getValue(dict_lower, 'department.documentnumber'),
               'cashcode': self.getValue(dict_lower, 'department.cashcode'),

            ################################ ВНИМАНИЕ ################################
            ################### Все поля добавляем в нижнем регистре #################
            #################### Наименование поля такое же как в БД #################
            ##########################################################################

               }

    def producePaymentDataObject(self,dict_lower):
        return{
               'paymentitemid': self.getValue(dict_lower, 'paymentitem.paymentitemid'),
               'paymentid': self.getValue(dict_lower, 'paymentitem.paymentid'),
               'creationtime': self.getValue(dict_lower, 'paymentitem.creationtime'),
               'accountnumber': self.getValue(dict_lower, 'paymentitem.accountnumber'),
               'amount': self.getValue(dict_lower, 'paymentitem.amount'),
               'commissionagent': self.getValue(dict_lower, 'paymentitem.commissionagent'),
               'commissionsubagent': self.getValue(dict_lower, 'paymentitem.commissionsubagent'),
               'terminalcode': self.getValue(dict_lower, 'paymentitem.terminalcode'),
               'terminaladdress': self.getValue(dict_lower, 'paymentitem.terminaladdress'),
               'providercode': self.getValue(dict_lower, 'paymentitem.providercode'),
               'providername': self.getValue(dict_lower, 'paymentitem.providername'),
               'providerinn': self.getValue(dict_lower, 'paymentitem.providerinn'),
               'providerphone': self.getValue(dict_lower, 'paymentitem.providerphone'),
               'provideraddress': self.getValue(dict_lower, 'paymentitem.provideraddress'),
               'categorycode': self.getValue(dict_lower, 'paymentitem.categorycode'),
               'categoryname': self.getValue(dict_lower, 'paymentitem.categoryname'),
               'agentcode': self.getValue(dict_lower, 'paymentitem.agentcode'),
               'agentname': self.getValue(dict_lower, 'paymentitem.agentname'),
               'agentinn': self.getValue(dict_lower, 'paymentitem.agentinn'),
               'agentphone': self.getValue(dict_lower, 'paymentitem.agentphone'),
               'agentaddress': self.getValue(dict_lower, 'paymentitem.agentaddress'),
               'subagentcode': self.getValue(dict_lower, 'paymentitem.subagentcode'),
               'subagentname': self.getValue(dict_lower, 'paymentitem.subagentname'),
               'subagentinn': self.getValue(dict_lower, 'paymentitem.subagentinn'),
               'subagentphone': self.getValue(dict_lower, 'paymentitem.subagentphone'),
               'subagentaddress': self.getValue(dict_lower, 'paymentitem.subagentaddress'),
               'cardnumber': self.getValue(dict_lower, 'paymentitem.cardnumber'),
               'cashcode': self.getValue(dict_lower, 'paymentitem.cashcode'),

            ################################ ВНИМАНИЕ ################################
            ################### Все поля добавляем в нижнем регистре #################
            #################### Наименование поля такое же как в БД #################
            ##########################################################################

               }

    def produceAspectDataObject(self, dict_lower):
        return{
               'aspectitemid': self.getValue(dict_lower, 'aspectitem.aspectitemid'),
               'goodsitemid': self.getValue(dict_lower, 'aspectitem.goodsitemid'),
               'aspectcode': self.getValue(dict_lower, 'aspectitem.aspectcode'),
               'aspectvaluecode': self.getValue(dict_lower, 'aspectitem.aspectvaluecode'),
               'cashcode': self.getValue(dict_lower, 'aspectitem.cashcode'),
               'productsectioncode': self.getValue(dict_lower, 'aspectitem.productsectioncode'),

            ################################ ВНИМАНИЕ ################################
            ################### Все поля добавляем в нижнем регистре #################
            #################### Наименование поля такое же как в БД #################
            ##########################################################################

               }

    def produceButtonDataObject(self,dict_lower):
        return{
               'buttonid': self.getValue(dict_lower, 'button.buttonid'),
               'code': self.getValue(dict_lower, 'button.code'),
               'name': self.getValue(dict_lower, 'button.name'),
               'tag': self.getValue(dict_lower, 'button.tag'),
               'rateobject': self.getValue(dict_lower, 'button.rateobject'),
               'ratetype': self.getValue(dict_lower, 'button.ratetype'),
               'ratevalue': self.getValue(dict_lower, 'button.ratevalue'),
               'maxratevalue': self.getValue(dict_lower, 'button.maxratevalue'),
               'minratevalue': self.getValue(dict_lower, 'button.minratevalue'),
               'inputvalue': self.getValue(dict_lower, 'button.inputvalue'),
               'cashcode': self.getValue(dict_lower, 'button.cashcode'),

            ################################ ВНИМАНИЕ ################################
            ################### Все поля добавляем в нижнем регистре #################
            #################### Наименование поля такое же как в БД #################
            ##########################################################################

               }

    def produceCertificateDataObject(self,dict_lower):
        return {
                'certificateid': self.getValue(dict_lower, 'certificate.certificateid'),
                'number': self.getValue(dict_lower, 'certificate.number'),
                'code': self.getValue(dict_lower, 'certificate.code'),
                'name': self.getValue(dict_lower, 'certificate.name'),
                'sum': self.getValue(dict_lower, 'certificate.sum'),
                'paymentsum': self.getValue(dict_lower, 'certificate.paymentsum'),
                'balance': self.getValue(dict_lower, 'certificate.balance'),
                'fullyspend': self.getValue(dict_lower, 'certificate.fullyspend'),

            ################################ ВНИМАНИЕ ################################
            ################### Все поля добавляем в нижнем регистре #################
            #################### Наименование поля такое же как в БД #################
            ##########################################################################

               }

    def produceGiftDataObject(self, dict_lower):
        return {
                'giftid': self.getValue(dict_lower, 'gift.giftid'),
                'documentid': self.getValue(dict_lower, 'gift.documentid'),
                'goodsitemid': self.getValue(dict_lower, 'gift.goodsitemid'),
                'bcode': self.getValue(dict_lower, 'gift.bcode'),
                'name': self.getValue(dict_lower, 'gift.name'),
                'isfiscal': self.getValue(dict_lower, 'gift.isfiscal')

            ################################ ВНИМАНИЕ ################################
            ################### Все поля добавляем в нижнем регистре #################
            #################### Наименование поля такое же как в БД #################
            ##########################################################################

               }

    def produceAlcoSetDataObject(self,dict_lower):
        return {
                'alcosetitemid': self.getValue(dict_lower, 'alcosetitem.alcosetitemid'),
                'goodsitemid': self.getValue(dict_lower, 'alcosetitem.goodsitemid'),
                'barcode': self.getValue(dict_lower, 'alcosetitem.barcode'),
                'code': self.getValue(dict_lower, 'alcosetitem.code'),
                'price': self.getValue(dict_lower, 'alcosetitem.price'),
                'excisemark': self.getValue(dict_lower, 'alcosetitem.excisemark'),
                'additionalexcisemark': self.getValue(dict_lower, 'alcosetitem.additionalexcisemark'),
                'alcocode': self.getValue(dict_lower, 'alcosetitem.alcocode'),
                'taracapacity': self.getValue(dict_lower, 'alcosetitem.taracapacity'),

            ################################ ВНИМАНИЕ ################################
            ################### Все поля добавляем в нижнем регистре #################
            #################### Наименование поля такое же как в БД #################
            ##########################################################################

               }

    def produceSupplierDataObject(self, dict_lower):
        return {
                'code': self.getValue(dict_lower, 'supplier.suppliercode'),
                'name': self.getValue(dict_lower, 'supplier.name'),
                'phone': self.getValue(dict_lower, 'supplier.phone'),
                'inn': self.getValue(dict_lower, 'supplier.inn'),
                'agentpaymentobject': self.getValue(dict_lower, 'supplier.agentpaymentobject')
               }

    def produceAgentsDataObject(self, dict_lower):
        return {
                'payingagentphones': self.getValue(dict_lower, 'paymentitem.agentphone'),
                'receivepaymentsoperatorphones': self.getValue(dict_lower, 'paymentitem.providerphone'),
                'transferoperatorname': self.getValue(dict_lower, 'paymentitem.subagentname'),
                'transferoperatorphone': self.getValue(dict_lower, 'paymentitem.subagentphone'),
                'transferoperatorinn': self.getValue(dict_lower, 'paymentitem.subagentinn'),
                'transferoperatoraddress': self.getValue(dict_lower, 'paymentitem.subagentaddress')
        }

    def produceConsultantDataObject(self, dict_lower):
        return {
            'code': self.getValue(dict_lower, 'consultant.code'),
            'name': self.getValue(dict_lower, 'consultant.name')
        }

    def produceMedicineDataObject(self, dict_lower):
        return {
            'code': self.getValue(dict_lower, 'medicine.code'),
            'party': self.getValue(dict_lower, 'medicine.party'),
            'series': self.getValue(dict_lower, 'medicine.series'),
            'shelflife': self.getValue(dict_lower, 'medicine.shelflife'),
            'producer': self.getValue(dict_lower, 'medicine.producer'),
            'inn': self.getValue(dict_lower, 'medicine.inn'),
            'country': self.getValue(dict_lower, 'medicine.country'),
            'options': self.getValue(dict_lower, 'medicine.options')
        }

    def produceFiscalDocumentDataObject(self, dict_lower):
        return {
            'fiscaldocumentid': self.getValue(dict_lower, 'fiscaldocument.fiscaldocumentid'),
            'documentid': self.getValue(dict_lower, 'fiscaldocument.documentid'),
            'fnnumber': self.getValue(dict_lower, 'fiscaldocument.fnnumber'),
            'serialnum': self.getValue(dict_lower, 'fiscaldocument.serialnum'),
            'fdnumber': self.getValue(dict_lower, 'fiscaldocument.fdnumber'),
            'shiftnumber': self.getValue(dict_lower, 'fiscaldocument.shiftnumber'),
            'taxsystem': self.getValue(dict_lower, 'fiscaldocument.taxsystem'),
            'hasofdticket': self.getValue(dict_lower, 'fiscaldocument.hasofdticket'),
            'sum': self.getValue(dict_lower, 'fiscaldocument.sum'),
            'fndate': self.getValue(dict_lower, 'fiscaldocument.fndate'),
            'ofddate': self.getValue(dict_lower, 'fiscaldocument.ofddate'),
            'fiscalsign': self.getValue(dict_lower, 'fiscaldocument.fiscalsign'),
            'ofdfiscalsign': self.getValue(dict_lower, 'fiscaldocument.ofdfiscalsign'),
            'updatetime': self.getValue(dict_lower, 'fiscaldocument.updatetime'),
            'tlv': self.getValue(dict_lower, 'fiscaldocument.tlv'),

            ################################ ВНИМАНИЕ ################################
            ################### Все поля добавляем в нижнем регистре #################
            #################### Наименование поля такое же как в БД #################
            ##########################################################################

               }
    def unloadSaleObjects(self):
        i = -1
        for i, row in enumerate(self.cdata):
            dict_lower = CaseInsensitiveDict(row)
            cdata = None
            cdata = self.produceCDataObject(dict_lower)
            documentid = str(self.getValue(dict_lower, 'document.documentid'))
            cdata["stornoPositions"] = [ p for p in self.unloadTSData(documentid) ]
            cdata["inventPositions"] = [ p for p in self.unloadTData(documentid) ]
            cdata["moneyPositions"] = [ p for p in self.unloadMData(documentid) ]
            cdata["failedMoneyPositions"] = [ p for p in self.unloadFailedMData(documentid) ]
            cdata['bonusPositions'] = [pos for pos in self.unloadBonusDataCheck(documentid)]
            cdata['cardPositions'] = [pos for pos in self.unloadCardDataCheck(documentid)]
            cdata['couponPositions'] = [pos for pos in self.unloadCouponData(documentid)]
            cdata['deptPositions'] = [pos for pos in self.unloadDeptData(documentid)]
            cdata['clientPositions'] = [pos for pos in self.unloadClientData(documentid)]
            cdata['paymentPositions'] = [pos for pos in self.unloadPaymentData(documentid)]
            cdata['buttonPositions'] = [pos for pos in self.unloadButtonData(documentid)]
            cdata['certificatePositions'] = [pos for pos in self.unloadCertificateData(documentid)]
            cdata['giftPositions'] = [pos for pos in self.unloadGiftData(documentid)]
            cdata['fiscaldocument'] = [pos for pos in self.unloadFiscalDocumentData(documentid)]
            yield cdata
        self.log.info("total: %d" % (i+1))

    # Выгрузка всех документов для указанных смен
    def unloadDocuments(self, shifts):
        if shifts:
            self.log.info('unload documents for shifts %s' % (shifts,))
            self.cdata.execute("select `document`.*, `workshift`.`shiftnum`, `workshift`.`shifttype` from `document`, `workshift` where \
                `document`.`workshiftid` IN %s and `workshift`.`workshiftid` = `document`.`workshiftid` and `closed` IN (1, 2)", (shifts,))
            for o in self.unloadSaleObjects():
                yield o

    # Получение измененных документов для указанных смен
    def getChangedDocuments(self, shifts):
        if shifts:
            self.log.info('get changed documents')
            self.cursor.execute("select `documentid` from `document` where `document`.`workshiftid` IN %s and \
                `document`.`changed` = 1 and `closed` IN (1, 2)", (shifts,))
            return self.cursor.fetchall()
        return ()

    # Выгрузка измененных документов
    def unloadChangedDocuments(self, documentIds):
        if documentIds:
            self.log.info('unload changed documents')
            self.cdata.execute("select `document`.*, `workshift`.`shiftnum`, `workshift`.`shifttype` from `document`, `workshift` \
                where `document`.`documentid` IN %s and `workshift`.`workshiftid`=`document`.`workshiftid` and `document`.`changed`=1 \
                and `closed` IN (1, 2)", (documentIds,))
            for o in self.unloadSaleObjects():
                yield o

    # Выгрузка документов для указанных смен по диапазону времени
    def unloadDocumentsByDate(self, shifts, fromDate, toDate):
        if not shifts:
            return

        forcedids = []
        
        if fromDate:
            # на кассе и КС разные названия поля с временем изменения записи в таблице fiscaldocument
            timeFieldName = 'updatetime' if logger.location == 'cash' else 'update_time'
            query = 'SELECT documentid FROM fiscaldocument WHERE fiscaldocument.{timeField} > %s AND (%s is NULL OR fiscaldocument.{timeField} <= %s)'.format(timeField=timeFieldName)
            self.cdata.execute(query, (fromDate, toDate, toDate,))
        
            for dictParams in self.cdata:
                for param in dictParams.values():
                    forcedids.append(str(param))

        forcedids = tuple(forcedids)

        self.log.info("Выгрузка документов в диапазоне времени: %s - %s" % (fromDate, toDate))
        
        try:
            self.cdata.execute("SELECT document.*, workshift.shiftnum, workshift.shifttype "
                               "FROM document, workshift "
                                "WHERE (document.workshiftid IN %s "
                                "   AND workshift.workshiftid = document.workshiftid "
                                "   AND (%s IS NULL or document.time_end > %s) "
                                "   AND (%s IS NULL or document.time_end <= %s) "
                                "   AND closed IN (1, 2)) "
                                "   OR (workshift.workshiftid = document.workshiftid AND document.documentid IN %s) "
                               "ORDER BY document.time_end", 
                                (shifts, fromDate, fromDate, toDate, toDate, forcedids,))
            self.log.info(str(self.cdata.rowcount))
        except Exception as e:
            self.log.error("Ошибка при выполнении запроса: " + str(e))
            raise

        for o in self.unloadSaleObjects():
            yield o

    def unloadTData(self, documentid):
        self.tdata.execute('select * from `goodsitem` where `documentid`=%s', (documentid,))
        for row in self.tdata:
            dict_lower = CaseInsensitiveDict(row)
            tdata = self.produceTDataObject(dict_lower)
            tdata["taxes"] = []
            for i in range(5):
                tdata["taxes"].append({"taxCode": self.getValue(dict_lower, 'goodsitem.vatcode%s' % (i+1)),
                                       "taxRate": self.getValue(dict_lower, 'goodsitem.vatrate%s' % (i+1)),
                                       "taxSum": self.getValue(dict_lower, 'goodsitem.vatsum%s' % (i+1))})
            goodsitemid = self.getValue(dict_lower, 'goodsitem.goodsitemid')
            tdata['discountPositions'] = [pos for pos in self.unloadDiscData(goodsitemid)]
            tdata['bonusPositions'] = [pos for pos in self.unloadBonusData(goodsitemid)]
            tdata['aspectPosition'] = [pos for pos in self.unloadAspectData(goodsitemid)]
            tdata['alcosetPositions'] = [pos for pos in self.unloadAlcoSetData(goodsitemid)]
            tdata['supplier'] = self.unloadSupplierData(goodsitemid, False)
            tdata['agents'] = self.unloadAgentsData(goodsitemid)
            tdata['consultant'] = self.unloadConsultantData(goodsitemid, False)
            tdata['gift'] = self.unloadPositionGiftData(goodsitemid)
            tdata['medicine'] = self.unloadMedicineData(goodsitemid, False)
            yield tdata

    def unloadDiscData(self, goodsitemid):
        self.discdata.execute('select * from `discitem` where `goodsitemid`=%s', (goodsitemid,))
        for row in self.discdata:
            dict_lower = CaseInsensitiveDict(row)
            discdata = self.produceDiscDataObject(dict_lower)
            discitemid = self.getValue(dict_lower, 'discitem.discitemid')
            discdata['additionalprice'] = self.unloadAdditionalPriceData(discitemid)
            yield discdata

    def unloadBonusData(self, goodsitemid):
        self.bonusdata.execute('select * from `bonusitem` where `goodsitemid` = %s order by `posnum`', (goodsitemid,))
        for row in self.bonusdata:
            dict_lower = CaseInsensitiveDict(row)
            bonusObject = self.produceBonusDataObject(dict_lower)
            yield bonusObject

    def unloadAspectData(self, goodsitemid):
        self.aspectdata.execute('SELECT * FROM `aspectitem` WHERE `goodsitemid` = %s', (goodsitemid,))
        for row in self.aspectdata:
            dict_lower = CaseInsensitiveDict(row)
            aspectObject = self.produceAspectDataObject(dict_lower)
            yield aspectObject

    def unloadTSData(self, documentid):
        self.tsdata.execute('select * from `stornogoodsitem` where `documentid`=%s', (documentid,))
        for row in self.tsdata:
            dict_lower = None
            dict_lower = CaseInsensitiveDict(row)
            tsdata = self.produceTSDataObject(dict_lower)
            tsdata["taxes"] = []
            for i in range(5):
                tsdata["taxes"].append({"taxCode": self.getValue(dict_lower, 'stornogoodsitem.vatcode%s' % (i+1)),
                                    "taxRate": self.getValue(dict_lower, 'stornogoodsitem.vatrate%s' % (i+1)),
                                    "taxSum": self.getValue(dict_lower, 'stornogoodsitem.vatsum%s' % (i+1))})
            stornoItemId = self.getValue(dict_lower, 'stornogoodsitem.stornogoodsitemid')
            tsdata['supplier'] = self.unloadSupplierData(stornoItemId, True)
            tsdata['consultant'] = self.unloadConsultantData(stornoItemId, True)
            tsdata['medicine'] = self.unloadMedicineData(stornoItemId, True)
            yield tsdata

    def unloadMData(self, documentid):
        self.mdata.execute('select * from `moneyitem` where `documentid`=%s', (documentid,))
        for row in self.mdata:
            dict_lower = CaseInsensitiveDict(row)
            mdata = self.produceMDataObject(dict_lower)
            yield mdata

    def unloadFailedMData(self, documentid):
        self.failedmdata.execute('select * from `failedmoneyitem` where `documentid`=%s', (documentid,))
        for row in self.failedmdata:
            dict_lower = CaseInsensitiveDict(row)
            failedMData = self.produceFailedMDataObject(dict_lower)
            yield failedMData

    def unloadBonusDataCheck(self, documentid):
        self.bonusdata.execute('select * from `bonusitem` where `documentid`=%s and `goodsitemid` is null order by `posnum`',
                               (documentid,))
        for row in self.bonusdata:
            dict_lower = CaseInsensitiveDict(row)
            bonusObject = self.produceBonusDataObject(dict_lower)
            yield bonusObject

    def unloadCardDataCheck(self, documentid):
        self.carddata.execute('select * from `carditem` where `documentid`=%s', (documentid,))
        for row in self.carddata:
            dict_lower = CaseInsensitiveDict(row)
            yield self.produceCardDataObject(dict_lower)

    def unloadCouponData(self, documentid):
        self.coupondata.execute('select * from `couponitem` where `documentid`=%s', (documentid,))
        for row in self.coupondata:
            dict_lower = CaseInsensitiveDict(row)
            yield self.produceCouponDataObject(dict_lower)

    def unloadDeptData(self,documentid):
        self.deptdata.execute('SELECT * FROM `department` JOIN (SELECT DISTINCT `departmentid` '
                              'FROM `goodsitem` WHERE `documentid` = %s UNION ALL SELECT DISTINCT `departmentid` '
                              'FROM `stornogoodsitem` WHERE `documentid` = %s UNION ALL SELECT `departmentid` '
                              'FROM `document` WHERE `documentid` = %s) t ON `department`.`departmentid` = t.`departmentid`',
                              (documentid, documentid, documentid))
        for row in self.deptdata:
            dict_lower = CaseInsensitiveDict(row)
            deptObject = self.produceDeptDataObject(dict_lower)
            yield deptObject

    def unloadClientData(self,documentid):
        if logger.location == 'cash':
            self.clientdata.execute('SELECT * FROM `client` join (SELECT DISTINCT `clientitemid` FROM `carditem` WHERE `documentid` = %s \
                UNION ALL SELECT DISTINCT d2.`clientitemid` FROM `document` d1 join `department` d2 on d1.`departmentid`=d2.`departmentid` WHERE d1.`documentid`=%s \
                UNION ALL SELECT `clientitemid` FROM `document` WHERE `documentid` = %s \
                UNION ALL SELECT DISTINCT d3.`clientitemid` FROM `goodsitem` g join `department` d3 on g.`departmentid`=d3.`departmentid` WHERE g.`documentid`=%s \
                UNION ALL SELECT DISTINCT d3.`clientitemid` FROM `stornogoodsitem` s join `department` d3 on s.`departmentid`=d3.`departmentid` WHERE s.`documentid`=%s \
                ) t on `client`.`clientitemid` = t.`clientitemid`', (documentid,documentid,documentid,documentid,documentid))
        else:
            self.clientdata.execute('SELECT * FROM `client` WHERE documentid = %s', (documentid,))
        for row in self.clientdata:
            dict_lower = CaseInsensitiveDict(row)
            clientObejct = self.produceClientDataObject(dict_lower)
            yield clientObejct

    def unloadPaymentData(self, documentid):
        self.paymentdata.execute('SELECT * FROM `paymentitem` JOIN (SELECT DISTINCT `paymentitemid` FROM `goodsitem` \
            WHERE `documentid` = %s UNION ALL SELECT DISTINCT  `paymentitemid` FROM `stornogoodsitem` WHERE `documentid` = %s) t ON \
            `paymentitem`.`paymentitemid` = t.`paymentitemid`', (documentid, documentid))
        for row in self.paymentdata:
            dict_lower = CaseInsensitiveDict(row)
            paymentObject = self.producePaymentDataObject(dict_lower)
            yield paymentObject

    def unloadButtonData(self, documentid):
        self.buttondata.execute('SELECT * FROM `button` JOIN (SELECT DISTINCT `buttonid` FROM `goodsitem` \
            WHERE `documentid` = %s UNION ALL SELECT DISTINCT `buttonid` FROM `stornogoodsitem` \
            WHERE `documentid` = %s UNION ALL SELECT `buttonid` FROM `document` WHERE `documentid` = %s) t USING(`buttonid`)', 
            (documentid,documentid,documentid))
        for row in self.buttondata:
            dict_lower = CaseInsensitiveDict(row)
            buttonObject = self.produceButtonDataObject(dict_lower)
            yield buttonObject

    def unloadCertificateData(self, documentid):
        self.certdata.execute('select * from `certificate` where `documentid`=%s', (documentid,))
        for row in self.certdata:
            dict_lower = CaseInsensitiveDict(row)
            certdata = self.produceCertificateDataObject(dict_lower)
            yield certdata

    def unloadGiftData(self, documentid):
        self.giftdata.execute('SELECT * FROM `gift` WHERE `documentid` = %s AND `goodsitemid` IS NULL', (documentid,))
        for row in self.giftdata:
            yield self.produceGiftDataObject(CaseInsensitiveDict(row))

    def unloadAlcoSetData(self, goodsitemid):
        self.alcosetdata.execute('select * from `alcosetitem` where `goodsitemid`=%s', (goodsitemid,))
        for row in self.alcosetdata:
            dict_lower = CaseInsensitiveDict(row)
            alcosetdata = self.produceAlcoSetDataObject(dict_lower)
            yield alcosetdata

    def unloadSupplierData(self, goodsitemid, isStorno):
        self.supplierdata.execute('select * from `supplier` where %s=%s' % (('`stornogoodsitemid`' if isStorno else '`goodsitemid`'), '%s'), (goodsitemid,))
        for row in self.supplierdata:
            dict_lower = CaseInsensitiveDict(row)
            return self.produceSupplierDataObject(dict_lower)
        return None

    def unloadAgentsData(self, goodsitemid):
        # Данные об агентах(список телефонов платежного агента и список телефонов оператора по приему платежей)
        # сохраняются в неиспользуемую таблицу paymentitem в поля:
        # paymentid - goodsitemid
        # agentphone - список телефонов платежного агента
        # providerphone - список телефонов оператора по приему платежей
        self.agentsdata.execute('SELECT * FROM `paymentitem` WHERE `paymentid` = %s', (str(goodsitemid),))
        for row in self.agentsdata:
            dict_lower = CaseInsensitiveDict(row)
            return self.produceAgentsDataObject(CaseInsensitiveDict(row))
        return None

    def unloadMedicineData(self, goodsitemid, isStorno):
        self.medicinedata.execute('select * from `medicine` where %s=%s' % (('`stornogoodsitemid`' if isStorno else '`goodsitemid`'), '%s'), (goodsitemid,))
        for row in self.medicinedata:
            dict_lower = CaseInsensitiveDict(row)
            return self.produceMedicineDataObject(dict_lower)
        return None
    
    def unloadConsultantData(self, goodsitemid, isStorno):
        self.consultantdata.execute('select * from `consultant` where %s=%s' % (('`stornogoodsitemid`' if isStorno else '`goodsitemid`'), '%s'), (goodsitemid,))
        for row in self.consultantdata:
            dict_lower = CaseInsensitiveDict(row)
            return self.produceConsultantDataObject(dict_lower)
        return None

    def unloadPositionGiftData(self, goodsitemid):
        self.giftdata.execute('SELECT * FROM `gift` WHERE `goodsitemid` = %s', (goodsitemid,))
        for row in self.giftdata:
            return self.produceGiftDataObject(CaseInsensitiveDict(row))
        return None

    def unloadFiscalDocumentData(self, documentid):
        self.fiscaldocumentdata.execute('select * from `fiscaldocument` where `documentid`=%s', (documentid,))
        for row in self.fiscaldocumentdata:
            dict_lower = CaseInsensitiveDict(row)
            fiscaldocumentdata = self.produceFiscalDocumentDataObject(dict_lower)
            yield fiscaldocumentdata

    def unloadAdditionalPriceData(self, discitemid):
        self.additionalpricedata.execute('SELECT * FROM `additionalprice` WHERE `discitemid` = %s', (discitemid,))
        for row in self.additionalpricedata:
            return self.produceAdditionalPriceDataObject(CaseInsensitiveDict(row))
        return None
