# -*- coding: utf-8 -*-

from common import config, filesmodify, logger
from upload.fam.session.soapSessionService import SoapSessionService

from common import aifSoapClasses as soapCl
from common import aif_objects as aif
from upload.converter import soap
from common import parameters
from common.rest_state import RestState as rest_state
from upload.node.node import QueueInserterConfig

from base64 import b64encode
from datetime import datetime
import inspect
import os
import sys
import threading
import time
import traceback

from lxml import etree
from spyne import EventManager
from spyne.decorator import srpc
from spyne.model.complex import Array
from spyne.model.primitive import String, Boolean
from spyne.model.fault import Fault
from spyne.service import ServiceBase
from spyne.const.http import HTTP_401
from spyne.server import wsgi
from spyne.const.xml_ns import DEFAULT_NS
from spyne.application import Application
from spyne.protocol.soap import Soap11
from spyne.server.wsgi import WsgiApplication, WsgiMethodContext
from spyne.util.wsgi_wrapper import run_twisted
from wsgiref.simple_server import WSGIRequestHandler, ServerHandler, make_server


log = logger.getUploadLogger("soap")
fileLogName = 'dict_SOAP_debug'
tns = 'http://aif.soap.exchange.artix.su'
primitiveAuth = False
soapLogin = 'login'
soapPasswd = 'passwd'
maxCountBackupRequests = 0
actualCountDaysForBackupRequests = 3
sessionIdleTimeout = 300
aifExchangeService = None


def getLogFileName(shopCode, dictionaryId, call_start):
    logDir = logger.getLogDirectory()
    return '%s/%s_%s_%s_%s.log' % (logDir, fileLogName, shopCode, dictionaryId, call_start)


class SoapConverter(object):
    name = 'soap'
    dict_state_id = None

    def __init__(self, hotKeyByBarcode=False):
        self.hotKeyByBarcode = hotKeyByBarcode


class EmptyNode(object):
    def __init__(self, shopId, realShopCode, isCashRun):
        self.id = shopId
        self.shopCode = realShopCode
        self.pack_size = config.getInt('FILEQUEUE', 'queueChunkSize', 25000)
        self.inserter = QueueInserterConfig(queue=parameters.QueueParameters(
                config.getString('FILEQUEUE', 'fileQueueRoot', '/linuxcash/cash/data/queues' if isCashRun else '/opt/queues'),
                config.getInt('FILEQUEUE', 'queueChunkSize', 25000),
                'artix.loaddict.cash' if isCashRun else None))
        self.currentConverter = SoapConverter()
        self.extendedProperties = {}


class AifExchangeService(SoapSessionService):
    errorInCalling = 'Error at the stage of calling the load'

    timeoutNodeReread = 300
    isCashRun = False
    shopCodeIsShopId = False

    singleShopSession = True

    curNodes = []
    lastTimeReadNodes = 0

    def rereadNodes(self):
        try:
            nodeGetter = __import__('upload.node.%s.nodeGetter' % (config.getString('UPLOAD', 'nodeGetter', 'cs30')), fromlist='q')
            nodeGetter.defaultNesServer = self.default_nes_server
            nodeGetter.defaultStatusServer = self.default_status_server
            AifExchangeService.curNodes = nodeGetter.getNodes()
            AifExchangeService.lastTimeReadNodes = time.time()
            log.info(u'ноды = %s' % ' '.join(str(e) for e in AifExchangeService.curNodes))
        except Exception as e:
            log.warning(u'Ошибка считывания нод. %s' % str(e))

    def checkShopCode(self, shopCode):
        if not shopCode or shopCode.strip() == '':
            return False, shopCode
        if time.time() - self.lastTimeReadNodes > self.timeoutNodeReread:
            self.rereadNodes()
        if self.curNodes:
            if not self.shopCodeIsShopId:
                ids = [n.id for n in self.curNodes if str(n.shopCode) == shopCode]
                if not ids:
                    log.warning(u'По коду %s не найдено ни одного магазина.' % shopCode)
                    return False, shopCode
                elif len(ids) == 1:
                    shopCode = ids[0]
                else:
                    log.warning(u'По коду %s найдено несколько магазинов. Невозможно однозначно определить целевой магазин.' % shopCode)
                    return False, shopCode
            elif not str(shopCode) in [str(node.id) for node in self.curNodes]:
                return False, shopCode
        elif not self.shopCodeIsShopId:
            log.warning(u'Невозможно найти по коду магазина его идентификатор, т.к. ноды не считаны.')
            return False, shopCode
        return True, shopCode

    def getPrestartResult(self, shopCode):
        dictConvertResult = soapCl.DictConvertResult()
        dictConvertResult.result = False
        dictConvertResult.info = self.errorInCalling
        res, shopId = self.checkShopCode(shopCode)
        if not res:
            dictConvertResult.info = 'Shop with code = %s does not exist.' % shopId
        return dictConvertResult, shopId

    def getNode(self, shopId, realShopCode, dictionaryId=None):
        node = EmptyNode(shopId, realShopCode, self.isCashRun)
        # nesServer и statusServer должен был бы задаться в nodeGetter, но тут используется своя EmptyNode
        node.currentConverter.statusServer = self.default_status_server
        node.nesServer = self.default_nes_server
        if not dictionaryId:
            dictionaryId = 'soap' + str(time.time())
        try:
            node.currentConverter.dict_state_id = rest_state.sendDictState(node, 'Start convert dict',
                    u'Начало конвертирования справочника.', rest_state.STATUS_DURING, dictionaryId)
        except Exception as e:
            log.warning("Can't write state to CS by REST. %s" % e)
        for n in self.curNodes:
            if str(n.id) == str(node.id):
                node.inserter = n.inserter
                for converter in n.converters:
                    if converter.name == 'aif':
                        node.currentConverter.hotKeyByBarcode = converter.hotKeyByBarcode
                break
        return node

    def getMethodName(self, callerMethod):
        if callerMethod.startswith('addExciseMarksWhites'):
            return 'addMultipleCmd'
        elif callerMethod.startswith('add'):
            return 'addCmd'
        elif callerMethod.startswith('deleteByDate'):
            return 'deleteCmdByDate'
        elif callerMethod.startswith('deleteOverdue'):
            return 'deleteCmdOverdue'
        elif callerMethod.startswith('deleteOutdated'):
            return 'deleteCmdOutdated'
        elif callerMethod.startswith('deleteFuture'):
            return 'deleteCmdFuture'
        elif callerMethod.startswith('deleteCatalogsAndTmcCatalogs'):
            return 'deleteCmdCatalogAndTmcCatalog'
        elif callerMethod.startswith('deleteTmcSuppliersBySomeField'):
            return 'deleteCmdBySomeField'
        elif callerMethod.startswith('delete'):
            return 'deleteCmd'
        elif callerMethod.startswith('update'):
            return 'updateCmd'
        return

    def startSession(self, shopCode, dictionaryId):
        realShopCode = shopCode
        result, shopId = self.getPrestartResult(shopCode)
        if result.info != self.errorInCalling:
            log.info(result.info)
            return result
        node = self.getNode(shopId, realShopCode, dictionaryId)
        temp_result = self.start_session(dictionaryId, node)
        result.info = temp_result.get("info")
        result.result = temp_result.get("success")
        return result

    def callCommand(self, aifClazz, soapObjects, shopCode='All', dictionaryId=None, clear=False, clearIfEmptyData=False):
        node = None
        soapConverter = None
        realShopCode = shopCode
        result, shopId = self.getPrestartResult(shopCode)
        if result.info != self.errorInCalling:
            log.info(result.info)
            return result
        if self.useSession:
            if not self.is_open_session(shopId, dictionaryId):
                result.result = False
                result.info = "Can not add package. Session for shop %s and dictionaryId %s is not opened" % (realShopCode, dictionaryId)
                log.warning(result.info)
                return result
            log.info('Add package in session %s for dictionaryId %s' % (AifExchangeService.sessions[self.get_session_id(shopId, dictionaryId)].node.currentConverter.dict_state_id, dictionaryId))
            result.result = True
            AifExchangeService.sessions[self.get_session_id(shopId, dictionaryId)].timeActive = time.time()
            soapConverter = AifExchangeService.sessions[self.get_session_id(shopId, dictionaryId)].converter
        else:
            node = self.getNode(shopId, realShopCode, dictionaryId)
            soapConverter = soap.SoapConverter(node)
            result = soapConverter.start()
            if not result.result:
                return result
            log.info('Loading dictionary id=%s' % dictionaryId)
        result = soapConverter.convert(soapObjects, shopId, aifClazz, self.getMethodName(inspect.stack()[1][3]), clear, clearIfEmptyData) if result.result else result
        if not self.useSession:
            if result.result:
                result = soapConverter.stop()
            else:
                soapConverter.stop()
        return result

    def stopSession(self, shopCode, dictionaryId):
        result, shopId = self.getPrestartResult(shopCode)
        if result.info != self.errorInCalling:
            log.info(result.info)
            return result
        temp_result = self.stop_session(shopCode, dictionaryId, shopId)
        result.info = temp_result.get("info")
        result.result = temp_result.get("success")
        return result

    def rollbackSession(self, shopCode, dictionaryId, sessionIdle=False):
        result, shopId = self.getPrestartResult(shopCode)
        if result.info != self.errorInCalling:
            log.info(result.info)
            return result
        temp_result = self.rollback_session(shop_code=shopCode, dictionary_id=dictionaryId, session_idle=sessionIdle, shop_id=shopId)
        result.info = temp_result.get("info")
        result.result = temp_result.get("success")
        return result


class AifSoapMethods(ServiceBase):
    __namespace__ = DEFAULT_NS

    """
    Список классов, наименования функций добавления и удаления
    для которых будет автоматически сгенерированы функции вставки и удаления по soap
    Классы в soap и aif должны называться одинаково.
    Например soapCl.Role, soapCl.RoleKey, aif.Role
    """
    listClasses = [
        ("ActionPanel", "ActionPanel", "ActionPanel", 'actionPanel', None),
        ("AdditionalCard", "AdditionalCard", "AdditionalCard", 'additionalcard', 'additionalcardKeys'),
        ("AdditionalPrice", None, None, None, None),
        ("Agent", None, None, 'objects', 'objectKeys'),
        ("Agerestrict", None, None, None, None),
        ("AlcoSet", "AlcoSet", "AlcoSet", 'alcoset', 'alcosetKeys'),
        ("Aspect", None, None, None, None),
        ("AspectScheme", None, None, 'aspectschemes', None),
        ("AspectValue", None, None, 'aspectvalues', None),
        ("AspectValueSet", None, None, 'aspectvaluesets', None),
        ("BackReason", None, None, None, None),
        ("Barcode", None, None, None, None),
        ("Button", None, None, None, None),
        ("Card", None, None, None, None),
        ("CardGroup", None, None, None, None),
        ("CardGroupSearchPlace", "CardGroupSearchPlace", "CardGroupSearchPlace", 'cardgroupsearchplace', 'cardgroupsearchplaceKeys'),
        ("CashConfig", "CashConfig", "CashConfig", 'cashconfig', 'cashconfigKeys'),
        ("CashUser", None, None, None, None),
        ("Catalog", None, None, None, None),
        ("CatalogAttribute", None, None, 'objects', 'objectKeys'),
        ("Category", "Categories", "Categories", 'objects', 'objectKeys'),
        ("Certificate", None, None, None, None),
        ("Client", None, None, None, None),
        ("Cmaction", None, None, 'objects', 'objectKeys'),
        ("Command", None, None, None, None),
        ("Consultant", None, None, None, None),
        ("CorrectionReason", None, None, 'correctionReason', None),
        ("Country", "Country", "Country", 'country', None),
        ("Currency", "Currencies", "Currencies", 'objects', 'objectKeys'),
        ("Dept", None, None, None, None),
        ("DiscountAction", None, None, None, None),
        ("DiscountsAuto", None, None, None, None),
        ("Event", None, None, None, None),
        ("HotKey", None, None, None, None),
        ("Invent", None, None, None, None),
        ("InventGroup", None, None, None, None),
        ("KbKey", None, None, None, None),
        ("Kit", None, None, None, None),
        ("KitDisc", None, None, None, None),
        ("KitInvent", None, None, None, None),
        ("LDiscAccumulate", None, None, None, None),
        ("LDiscCard", None, None, None, None),
        ("LDiscFixed", None, None, None, None),
        ("LDiscScheme", None, None, None, None),
        ("LVat", None, None, None, None),
        ("MCashUser", None, None, None, None),
        ("Macro", None, None, 'marcos', None),
        ("Measure", None, None, None, None),
        ("Medicine", "Medicine", "Medicine", 'medicine', None),
        ("Menu", "Menu", "Menu", 'objects', 'objectKeys'),
        ("Organization", None, None, None, None),
        ("Picklist", None, None, None, None),
        ("Price", None, None, None, 'pricesKeys'),
        ("RemainBarcode", None, None, 'remainbarcodes', 'remainbarcodeKeys'),
        ("Role", None, None, 'objects', 'objectKeys'),
        ("SaleRestrict", None, None, 'objects', 'objectKeys'),
        ("Shop", None, None, None, None),
        ("SoftCheck", None, None, 'softchecks', 'softcheckKeys'),
        ("Supplier", None, None, None, None),
        ("TaxGroup", None, None, None, None),
        ("TimePeriod", None, None, None, None),
        ("Timerestrict", None, None, None, None),
        ("Tmc", "Tmc", "Tmc", 'objects', 'objectKeys'),
        ("TmcRequiredQuantity", "TmcRequiredQuantities", "TmcRequiredQuantities", 'objects', 'objectKeys'),
        ("TmcScale", None, None, None, None),
        ("TmcScaleGroup", None, None, None, None),
        ("TmcVisualVerify", "TmcVisualVerifies", "TmcVisualVerifies", 'objects', 'objectKeys'),
        ("Unit", None, None, None, None),
        ("Verification", None, None, None, None),
        ]

    def createAddFunction(className, funcName, addObjName):
        return """
@srpc(Array(soapCl.%s),
      String(default='All', min_occurs=1, nillable=False),
      Boolean(default=False),
      Boolean(default=False),
      String(encoding='utf8'),
      _returns=soapCl.DictConvertResult)
def %s(%s, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
    return aifExchangeService.callCommand(aif.%s, %s, shopCode, dictionaryId, clear, clearIfEmptyData)
    """ % (className, "add" + (funcName if funcName is not None else className + "s"), addObjName, className, addObjName)

    def createDeleteFunction(className, funcName, delObjName):
        return """
@srpc(Array(soapCl.%sKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
def %s(%s, shopCode = 'All', dictionaryId = None):
    return aifExchangeService.callCommand(aif.%s, %s, shopCode, dictionaryId)
        """ % (className, "delete" + (funcName if funcName is not None else className + "s"), delObjName, className, delObjName)

    # Создадим функции
    for className, addFuncName, deleteFuncName, addObjName, delObjName in listClasses:
        exec(createAddFunction(className, addFuncName, '%s%ss' % (className[0].lower(), className[1:]) if addObjName is None else addObjName))
        exec(createDeleteFunction(className, deleteFuncName, '%s%sKeys' % (className[0].lower(), className[1:]) if delObjName is None else delObjName))

    @srpc(String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def startSession(shopCode, dictionaryId):
        return aifExchangeService.startSession(shopCode, dictionaryId)

    @srpc(String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def stopSession(shopCode, dictionaryId):
        return aifExchangeService.stopSession(shopCode, dictionaryId)

    @srpc(String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def rollbackSession(shopCode, dictionaryId):
        return aifExchangeService.rollbackSession(shopCode, dictionaryId)

    @srpc(Array(soapCl.Accountpincode), String(default='All', min_occurs=1, nillable=False, values=['All']), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addAccountpincodes(accountpincodes, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Accountpincode, accountpincodes, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.AccountpincodeKey), String(default='All', min_occurs=1, nillable=False, values=['All']), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteAccountpincodes(accountpincodeKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Accountpincode, accountpincodeKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.CatalogKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteCatalogsAndTmcCatalogs(catalogKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Catalog, catalogKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.ChangeAccountBalance), String(default='All', min_occurs=1, nillable=False, values=['All']), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addChangeAccountBalances(changeAccountBalances, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.ChangeAccountBalance, changeAccountBalances, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.ChangeAccountBalanceKey), String(default='All', min_occurs=1, nillable=False, values=['All']), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteChangeAccountBalances(changeAccountBalanceKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.ChangeAccountBalance, changeAccountBalanceKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.ChangeCardAccount), String(default='All', min_occurs=1, nillable=False, values=['All']), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addChangeCardAccounts(changeCardAccounts, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.ChangeCardAccount, changeCardAccounts, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.ChangeCardAccountKey), String(default='All', min_occurs=1, nillable=False, values=['All']), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteChangeCardAccounts(changeCardAccountKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.ChangeCardAccount, changeCardAccountKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.ExciseMark), String(default='All', min_occurs=1, nillable=False, values=['All']), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addExciseMarks(depts, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.ExciseMark, depts, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.ExciseMarkKey), String(default='All', min_occurs=1, nillable=False, values=['All']), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteExciseMarks(deptKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.ExciseMark, deptKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.ExciseMarkWhite), String(default='All', min_occurs=1, nillable=False, values=['All']), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addExciseMarkWhites(depts, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.ExciseMarkWhite, depts, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.ExciseMarksWhite), String(default='All', min_occurs=1, nillable=False, values=['All']), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addExciseMarksWhites(depts, shopCode='All', clear=False, clearIfEmptyData=False, dictionaryId=None):
        return aifExchangeService.callCommand(aif.ExciseMarkWhite, depts, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.ExciseMarkWhiteKey), String(default='All', min_occurs=1, nillable=False, values=['All']), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteExciseMarkWhites(deptKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.ExciseMarkWhite, deptKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.InventCatalog), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addInventCatalogs(inventCatalogs, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.InventCatalog, inventCatalogs, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.InventCatalog), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteInventCatalogs(inventCatalogs, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.InventCatalog, inventCatalogs, shopCode, dictionaryId)

    @srpc(Array(soapCl.PriceKeyByDate), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteByDatePrices(priceKeysByDate, shopCode='All', dictionaryId=None):
        return aifExchangeService.callCommand(aif.Price, priceKeysByDate, shopCode, dictionaryId)

    @srpc(Array(soapCl.OverduePriceKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteOverduePrices(overduePriceKeys, shopCode='All', dictionaryId=None):
        return aifExchangeService.callCommand(aif.Price, overduePriceKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Remain), String(default='?', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addRemains(remains, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Remain, remains, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.RemainKey), String(default='?', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteRemains(remainsKeys, shopCode='All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Remain, remainsKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.OutdatedRemainKey), String(default='?', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteOutdatedRemains(outdatedRemainKeys, shopCode='All', dictionaryId=None):
        return aifExchangeService.callCommand(aif.Remain, outdatedRemainKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.AdditionalPriceKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteOutdatedAdditionalPrices(additionalPriceKeys, shopCode='All', dictionaryId=None):
        return aifExchangeService.callCommand(aif.AdditionalPrice, additionalPriceKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.AdditionalPriceKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteFutureAdditionalPrices(additionalPriceKeys, shopCode='All', dictionaryId=None):
        return aifExchangeService.callCommand(aif.AdditionalPrice, additionalPriceKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.HotKeyInvent), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addHotKeyInvents(hotKeyInvents, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.HotKeyInvent, hotKeyInvents, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.HotKeyInvent), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteHotKeyInvents(hotKeyInvents, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.HotKeyInvent, hotKeyInvents, shopCode, dictionaryId)

    @srpc(Array(soapCl.AlcoCode), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addAlcoCodes(alcocodes, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.AlcoCode, alcocodes, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.AlcoCode), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteAlcoCodes(alcocodes, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.AlcoCode, alcocodes, shopCode, dictionaryId)

    @srpc(Array(soapCl.TmcAlcocode), String(default='All', min_occurs=1, nillable=False, values=['All']), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addTmcAlcocodes(tmcAlcocodes, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.TmcAlcocode, tmcAlcocodes, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.TmcAlcocode), String(default='All', min_occurs=1, nillable=False, values=['All']), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteTmcAlcocodes(tmcAlcocodes, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.TmcAlcocode, tmcAlcocodes, shopCode, dictionaryId)

    @srpc(Array(soapCl.AspectValueSetValues), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addAspectValueSetValues(aspectvaluesetvalues, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.AspectValueSetValues, aspectvaluesetvalues, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.AspectValueSetValues), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteAspectValueSetValues(aspectvaluesetvalues, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.AspectValueSetValues, aspectvaluesetvalues, shopCode, dictionaryId)

    @srpc(Array(soapCl.CardGroupVerification), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addCardGroupVerifications(cardgroupverifications, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.CardGroupVerification, cardgroupverifications, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.CardGroupVerification), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteCardGroupVerifications(cardgroupverifications, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.CardGroupVerification, cardgroupverifications, shopCode, dictionaryId)

    @srpc(Array(soapCl.TmcSupplier), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addTmcSuppliers(tmcsuppliers, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.TmcSupplier, tmcsuppliers, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.TmcSupplier), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteTmcSuppliers(tmcsuppliers, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.TmcSupplier, tmcsuppliers, shopCode, dictionaryId)

    @srpc(Array(soapCl.TmcSupplierForDelBySomeField), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteTmcSuppliersBySomeField(tmcsuppliers, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.TmcSupplier, tmcsuppliers, shopCode, dictionaryId)

    @srpc(Array(soapCl.RemainTmc), String(default='All', min_occurs=1, nillable=False, values=['All']), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addRemainTmcs(remaintmcs, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.RemainTmc, remaintmcs, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.RemainTmcKey), String(default='All', min_occurs=1, nillable=False, values=['All']), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteRemainTmcs(remaintmcKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.RemainTmc, remaintmcKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.PersonalOffer), String(default='All', min_occurs=1, nillable=False, values=['All']), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addPersonalOffer(personaloffer, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.PersonalOffer, personaloffer, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.PersonalOfferKey), String(default='All', min_occurs=1, nillable=False, values=['All']), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deletePersonalOffer(personalofferKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.PersonalOffer, personalofferKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.CatalogRestrict), String(default='All', min_occurs=1, nillable=False, values=['All']), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addCatalogRestrict(catalogrestricts, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.CatalogRestrict, catalogrestricts, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.CatalogRestrict), String(default='All', min_occurs=1, nillable=False, values=['All']), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteCatalogRestrict(catalogrestricts, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.CatalogRestrict, catalogrestricts, shopCode, dictionaryId)

    @srpc(Array(soapCl.MCashUserAccess), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addMCashUserAccesses(mcashuseraccesses, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.MCashUserAccess, mcashuseraccesses, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.MCashUserAccess), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteMCashUserAccesses(mcashuseraccesses, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.MCashUserAccess, mcashuseraccesses, shopCode, dictionaryId)


### !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
### Добавление новых методов нужно делать через listClasses (например добавление товара ("Tmc", "addTmc", "deleteTmc"))
### Это быстрее, меньше шансов ошибиться и меньше кода
### !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


class AifEventManager(EventManager):
    def getLogName(self, ctx):
        if ctx and ctx.in_object:
            return getLogFileName(ctx.in_object.shopCode, ctx.in_object.dictionaryId, ctx.call_start)
        elif ctx:
            return getLogFileName('UNCKNOWN_SHOP', 'UNCKNOWN_DICT_ID', ctx.call_start)
        else:
            return getLogFileName('UNCKNOWN_SHOP', 'UNCKNOWN_DICT_ID', time.time()*1000)

    def fire_event(self, event_name, ctx):
        if event_name == 'aif_log_request':
            with open(self.getLogName(ctx), 'a') as f:
                f.write('%s Request:\n' % datetime.now())
                f.write(etree.tostring(ctx.in_document, pretty_print=True).decode())
        if event_name == 'wsgi_close':
            logFile = self.getLogName(ctx)
            with open(logFile, 'a') as f:
                f.write('%s Response:\n' % datetime.now())
                f.write(etree.tostring(ctx.out_document, pretty_print=True).decode())
            if ctx and ctx.in_object:
                backupRequest(ctx.in_object.shopCode, [logFile])
            else:
                backupRequest('UNCKNOWN_SHOP', [logFile])
        EventManager.fire_event(self, event_name, ctx)


class AifWsgiApplication(WsgiApplication):
    def __init__(self, app, chunked=True, max_content_length=2 * 1024 * 1024, block_length=8 * 1024):
        super(AifWsgiApplication, self).__init__(app, chunked, max_content_length, block_length)
        self.event_manager = AifEventManager(self)
        if self.doc.wsdl11 is not None:
            self.doc.wsdl11.event_manager.add_listener("wsdl_document_built", self.on_wsdl_document_built)

    # ставим тег с сервисами в самый конец, чтобы формировалась корректная wsdl
    def on_wsdl_document_built(self, doc):
        services = doc.root_elt.findall('{http://schemas.xmlsoap.org/wsdl/}service')
        for service in services:
            doc.root_elt.remove(service)
            doc.root_elt.append(service)

    def __call__(self, req_env, start_response, wsgi_url=None):
        print(req_env)
        if primitiveAuth:
            userAndPass = b64encode(b"%s:%s" % (soapLogin, soapPasswd)).decode("ascii")
            if req_env.get('HTTP_AUTHORIZATION') != 'Basic %s' % userAndPass:
                ctx = WsgiMethodContext(self, req_env, 'text/xml; charset=utf-8')
                start_response(HTTP_401, wsgi._gen_http_headers(ctx.transport.resp_headers))
                return [HTTP_401]
        return WsgiApplication.__call__(self, req_env, start_response, wsgi_url=None)


class AifApplication(Application):
    def __init__(self, services, tns, name=None, in_protocol=None, out_protocol=None, interface=None):
        Application.__init__(self, services, tns, name, in_protocol, out_protocol, interface)
        self.event_manager = AifEventManager(self)

    def process_request(self, ctx):
        self.event_manager.fire_event('aif_log_request', ctx)
        Application.process_request(self, ctx)

    def call_wrapper(self, ctx):
        try:
            return Application.call_wrapper(self, ctx)
        except Exception as e:
            raise Fault(faultstring=e)


class ServerHandlerAif(ServerHandler):
    def error_output(self, environ, start_response):
        try:
            start_response(self.error_status, self.error_headers[:], sys.exc_info())
            exc_type, exc_value, exc_traceback = sys.exc_info()
            mess = traceback.format_exception_only(exc_type, exc_value)
        except Exception as e:
            print(e)
        result = []
        result.append('''<?xml version='1.0' encoding='UTF-8'?>''')
        result.append('''
<senv:Envelope xmlns:tns="%s" xmlns:senv="http://schemas.xmlsoap.org/soap/envelope/">
  <senv:Body>
    <tns:Response>
      <tns:Result>
        <tns:info>%s</tns:info>
        <tns:result>false</tns:result>
      </tns:Result>
    </tns:Response>
  </senv:Body>
</senv:Envelope>''' % (tns, mess))
        return result


class WSGIRequestHandlerAif(WSGIRequestHandler):

    timeout = 300  # connection timeout

    def handle(self):
        self.raw_requestline = self.rfile.readline()
        print(self.raw_requestline)
        if not self.parse_request():  # An error code has been sent, just exit
            return
        if primitiveAuth:
            userAndPass = b64encode(b"%s:%s" % (soapLogin, soapPasswd)).decode("ascii")
            if self.headers.getheader('Authorization') != 'Basic %s' % userAndPass:
                self.send_response(401, message='Authorization error.')
                return
        handler = ServerHandlerAif(
            self.rfile, self.wfile, self.get_stderr(), self.get_environ()
        )
        handler.request_handler = self
        handler.run(self.server.get_app())


def backupRequest(shopCode, logFiles=None):
    logDir = logger.getLogDirectory()
    if not logFiles:
        logFiles = filesmodify.getFilesByMask(logDir, ['%s_%s*' % (fileLogName, shopCode)])
    for f in logFiles:
        filesmodify.archive(log, os.path.dirname(f), os.path.join(logDir, 'SOAP_BACKUP', shopCode),
                maxCountFiles=maxCountBackupRequests, actualCountDays=actualCountDaysForBackupRequests,
                removeSRC=True, fileName=os.path.basename(f), loginfo=False)


def sessionThread():
    try:
        while True:
            sessions = AifExchangeService.sessions
            for k in list(sessions):
                if (time.time() - sessions[k].timeActive) > sessionIdleTimeout:
                    aifExchangeService.rollbackSession(sessions[k].node.id, sessions[k].dictId, sessionIdle=True)
            time.sleep(2)
    except Exception as e:
        log.error(e)


def start(isCash=False):
    global primitiveAuth
    global soapLogin
    global soapPasswd
    global maxCountBackupRequests
    global actualCountDaysForBackupRequests
    global sessionIdleTimeout
    global aifExchangeService

    AifExchangeService.timeoutNodeReread = config.getInt('UPLOAD', 'timeoutNodeReread', 300)
    AifExchangeService.isCashRun = isCash
    AifExchangeService.shopCodeIsShopId = config.getBool('SOAP_UPLOAD', 'shopCodeIsShopId', False)
    AifExchangeService.useSession = config.getBool('SOAP_UPLOAD', 'useSession', False)
    AifExchangeService.singleShopSession = config.getBool('SOAP_UPLOAD', 'singleShopSession', False)

    useValidation = config.getBool('SOAP_UPLOAD', 'useValidation', False)

    maxContentLength = config.getInt('SOAP_UPLOAD', 'maxContentLength', (8 * 1024 * 1024) * 10)
    primitiveAuth = config.getBool('SOAP_UPLOAD', 'authorize', False)
    soapLogin = config.getString('SOAP_UPLOAD', 'login', 'admin')
    soapPasswd = config.getString('SOAP_UPLOAD', 'passwd', 'admin')
    maxCountBackupRequests = config.getInt('SOAP_UPLOAD', 'maxCountBackupRequests', 0)
    actualCountDaysForBackupRequests = config.getInt('SOAP_UPLOAD', 'actualCountDaysForBackupRequests', 3)
    sessionIdleTimeout = config.getInt('SOAP_UPLOAD', 'sessionIdleTimeout', 300)

    port = config.getInt('SOAP_UPLOAD', 'port', 18080)
    useMultiprocessing = config.getBool('SOAP_UPLOAD', 'multiprocessing', False)

    defaultStatusServer = config.getString('CS', 'defaultStatusServer', 'localhost:8080')
    defaultNesServer= config.getString('CS', 'defaultNesServer', 'localhost:28081')

    aifExchangeService = AifExchangeService(is_cash=isCash, default_status_server=defaultStatusServer, default_nes_server=defaultNesServer)

    blockLength = 10 * 1024 * 1024  # 10 MB
    if blockLength > maxContentLength:
        blockLength = maxContentLength

    if AifExchangeService.useSession:
        log.info("Starting thread for idle session")
        thread = threading.Thread(target=sessionThread)
        thread.daemon = True
        thread.start()

    application = AifApplication([AifSoapMethods], tns, name='AifExchange',
             in_protocol=Soap11(validator='soft' if useValidation else None), out_protocol=Soap11())

    if useMultiprocessing:
        log.info("Starting multiprocessing: exchangeUploadSoapService")
        if sys.version_info.major == 3 and sys.version_info.minor >= 9:
            from upload.fam.soapAiohttp import startService
            authString = b64encode(b"%s:%s" % (soapLogin, soapPasswd)).decode("ascii") if primitiveAuth else None
            startService(application, maxContentLength, port, authString)
        else:
            twisted_apps = [(AifWsgiApplication(application, chunked=True, max_content_length=maxContentLength, block_length=blockLength), 'soap')]
            run_twisted(twisted_apps, port)
    else:
        log.info("Starting single process exchangeUploadSoapService")
        wsgi_app = AifWsgiApplication(application, max_content_length=maxContentLength, block_length=blockLength)
        server = make_server('0.0.0.0', port, wsgi_app, handler_class=WSGIRequestHandlerAif)
        server.serve_forever()

    log.info("Stoped exchangeUploadSoapService")
