# -*- coding: UTF-8 -*-

import os
import sys
from common import logger
from upload.inserter import _Inserter
nesPath = '/opt/nes/src/mysql'
if logger.location == 'cash':
    nesPath = '/linuxcash/cash/exchangesystems/nes/src/mysql'

tmpPath = sys.path[0]
sys.path[0] = os.path.normpath(nesPath)
sys.path.insert(1, os.path.normpath(nesPath + '/..'))
import db_inserter
from nes_common import rest_client
import traceback
sys.path[0] = tmpPath


class Inserter(_Inserter):
    def __init__(self, node):
        _Inserter.__init__(self, node)
        self.loadDictStatusSender = rest_client.LoadDictStatusSender(node.id,
                dictCashLoadStatusId=node.currentConverter.dict_state_id,
                statusServer=node.currentConverter.statusServer,
                cashCode=node.cashCode,
                version=node.extendedProperties.get('version'))
        self.inserter = None
        self.countGoodCommands = {'add': 0, 'delete': 0, 'clear': 0, 'update': 0}
        self.countBadCommands = {'add': 0, 'delete': 0, 'clear': 0, 'update': 0}
        self.tableNames = set()
        self.transactionId = None  # Необходим для отображение процентов загрузки на кассе. Заполняется в aif, если вставка в БД напрямую

    def start(self):
        shop = {'clusterId': '', 'shopId': self.node.id}
        try:
            self.inserter = db_inserter.DB_Inserter(shop, loadDictStatusSender=self.loadDictStatusSender,
                    disableProgressFile=self.disableProgressFile)
            self.log.info(u'Подготовка к загрузке справочников...')
            self.inserter.start(self.transactionId)
        except Exception as e:
            if self.inserter:
                self.inserter.mysqlConnBlock.rollback()
                self.inserter.mysqlConnBlock.cursor().close()
                self.inserter.mysqlConnBlock.close()
            if e[0] == 1062:
                self.log.warning(u'БД для записи справочников на магазин %s заблокирована. %s' % (self.node.id, e))
                return False
            self.log.error(u'Ошибка работы с БД. %s' % e)
            return False
        return True

    def add_package(self, dictObjects):
        self.log.info(u'Добавляем объекты в БД')
        try:
            tables, countGoodCommandsInPackage, countBadCommansInPackage = self.inserter.add_package(dictObjects)
            for t in tables:
                self.tableNames.add(t)
            for key in self.countGoodCommands.keys():
                self.countGoodCommands[key] += countGoodCommandsInPackage[key]
                self.countBadCommands[key] += countBadCommansInPackage[key]
            self.objectsCount += len(dictObjects)
        except Exception as e:
            self.log.error(u'Ошибка при добавлении объектов на загрузку в БД: %s' % e)
            self.log.error(traceback.format_exc())
            return False
        return True
    
    def stop(self):
        try:
            self.log.info(u'Загрузка окончена. Разблокируем БД и закрываем соединение')
            mes_ru, mes_en = self.dictsCountCommandsToStr()
            result = self.inserter.stop()
            if not result:
                self.loadDictStatusSender.sendDictState('Stop failure insert into DB', u'Ошибка завершения вставки в БД', rest_client.STATUS_ERROR)
                self.tableNames.clear()
                return result
            if self.objectsCount == 0:
                self.loadDictStatusSender.sendDictState(u'No entries have been added to DB.', u'Ни одной записи не добавлено в БД.', rest_client.STATUS_ERROR)
            elif sum(self.countBadCommands.values()) > 0:
                self.loadDictStatusSender.sendDictState('Dictionary is not fully loaded. %s' % mes_en, u'Справочник загружен, но не полностью. Были ошибки. %s' % mes_ru, rest_client.STATUS_READY_WITH_ERROR, self.tableNames)
                if not self.loadDictStatusSender.cashCode: # при загрузке в КС для остальных таблиц, у которых была успешная загрузка ставим "хороший" статус
                    self.loadDictStatusSender.sendDictState(u'OK %s' % mes_en, u'ОК %s' % mes_ru, rest_client.STATUS_READY)
            else:
                self.loadDictStatusSender.sendDictState(u'OK %s' % mes_en, u'ОК %s' % mes_ru, rest_client.STATUS_READY)
            if self.node.currentConverter.dict_state_id and (self.objectsCount > 0 or self.node.extendedProperties.get('remoteLoadDictCommand', False)) and logger.location == 'cs':
                nesGenerateCommandMode = self.node.extendedProperties.get('nesGenerateCommandMode', 'increment')
                nesGenerateCommandCashesList = [x.strip() for x in self.node.extendedProperties.get('nesGenerateCommandCashesList', '').split(',')]
                unloadElementName = self.node.extendedProperties.get('unloadElementName', None)
                generateList = {}
                dictBackOfficeIds = set()
                if nesGenerateCommandMode != 'none':
                    if unloadElementName:
                        generateList = {nesGenerateCommandMode: {unloadElementName: nesGenerateCommandCashesList}}
                    else:
                        dictBackOfficeIds = set([self.node.currentConverter.dict_state_id])
                self.log.info('Запускаем генерацию справоников на КС')
                rest_client.generateDict(self.node.id, dictBackOfficeIds, generateList)
            self.tableNames.clear()
        except Exception as e:
            self.log.error(u'Ошибка при остановке загрузки объектов в БД: %s' % e)
            self.log.error(traceback.format_exc())
            return False
        return True

    def dictsCountCommandToStr(self, command, ruName):
        if self.countGoodCommands[command] + self.countBadCommands[command] > 0:
            s_ru = u'Команды %s: успешно - %s, ошибок - %s; ' % (ruName, self.countGoodCommands[command], self.countBadCommands[command])
            s_en = u'%s commans: ok - %s, error - %s; ' % (command, self.countGoodCommands[command], self.countBadCommands[command])
            return s_ru, s_en
        return '', ''

    def dictsCountCommandsToStr(self):
        clear_ru, clear_en = self.dictsCountCommandToStr('clear', u'очистки')
        add_ru, add_en = self.dictsCountCommandToStr('add', u'добавления')
        delete_ru, delete_en = self.dictsCountCommandToStr('delete', u'удаления')
        update_ru, update_en = self.dictsCountCommandToStr('update', u'обновления')
        return clear_ru + add_ru + delete_ru + update_ru, clear_en + add_en + delete_en + update_en
